$ codeset=UTF-8

$set 1 #Align

1 Дно Средина
2 Дно Лево
3 Дно Десно
4 Хоринзонтално
5 Лево 
6 Лево Дно
7 Лево Средина
8 Лево Горе
9 Приближно
10 Десно
11 Десно Дно
12 Десно Средина
13 Right Top
14 Горе Средина
15 Горе Лево
16 Горе Десно
17 Вертикално

$set 2 #BaseDisplay

1 Откажувам... складирам јадро\n
2 Исклучување \n
3 %s:      сигнал %d е фатен\n

$set 3 #Common

1 Алфа
2 Автоматски кријај
3 Состави
4 Опции на Составувач
5 Составувач
6 Верзија на Составувач
7  приклучување
8       команди
9           нлс
10         мени
20   windowмени
11         стил
12 Основни
13 оневозможени
14 Грешки
15 Fluxbox верзија
16 Maximize Over
17 GIT Надзор
18 Видлив
19 Во овој стил нема зададено подесување на позадината.\nВе молиме советувајте се со упатството или ЧПП.

$set 4 #Configmenu

1 Одмаглување
2 Авто кревање
3 Кликнувањето крева
4 Кликни за фокус
5 Украси ги привремените прозорци
6 Сменување на Раб. површина на тркалце
7 Фокусирај прозорец на смена на раб. место
8 Фокусен модел
9 Фокусирај Нови Прозорци
10 Целосно раширување
11 Image Dithering
12 Проѕирно преместување
14 Фокус на глушец
15 Скок до др. Раб. место
16 Присили лажна проѕирност
17 Алфа на мени
18 Проѕирност
19 Фокусиран алфа прозорец
20 дефокусиран алфа прозорец
21 Можности на јазиче
22 Јазиче над прозорец
23 Надворешна големина на јазиче
24 Фокусирај јазиче на клик
25 Фокус на јазиче при движење
26 Можност за низцело
27 Игнорирај промена на големина
28 Оневозможи Преместување
29 Оневозможи Променување големина

$set 5 #Ewmh

1 Опасност: Меморијата е полна, не може да се оддели За EWMH листата на клиенти

$set 6 #FbTkError

1 Предупредување! Висината > 3200, местам Висина = 3200
2 Предупредување! Ширината > 3200, местам Ширина = 3200
3 Предупредување: неможам да го вклучам резервниот фонт
4 Грешка при доделување.
5 Грешка во правење на пиксел
6 Неможе да се направи XImage
7 Неуспех при читање
8 Предупредување: Не успеав да пронајдам валиден формат за алфа.
9 Предупредување: Не успеав да создадам алфа слика.
10 Предупредување: Не успеав да создадам алфа икона.
11 Не успеав да пронајдам формат за (%d)
12 Меморијата е полна
13 Меморијата на синиот магацин се преполни.
14 Меморијата на зелениот магацин се преполни..
15 Меморијата на црвениот магацин се преполни..
16 Неуспеав да прочитам дел од темата
17 неподдржана графика
18 Поставување основна вредност

$set 7 #Fluxbox

1 rc фајлот е невалиден!
2 Не успеав да вчитам groupfile
3 Не успеав да вчитам датабаза
4 Пробувам Со
5 Предупредување ! Неможам да најдам екран за прозорецот да го мапирам!
6 Неможе да се направи директориумот %s
7 Неможам да пронајдам екрани за управување.\nПогрижете се да немате приклучено  уште еден управувач на прозорци.
8 Грешка во расленувањето на регуларниот израз
9 Катастрофа! Може само да има еден примерок од класата на fluxbox.
10 Неможам да се поврзам на X серверот.\nПогрижи се да стартува X пред Fluxbox.
11 Предупредување: X серверот не го поддржува одбраниот локал
12 Предупредување: неможам да поставам локални менувачи

$set 8 #Gnome

1 Опасност: Меморијата е полна, не може да се оддели за Гном клиент листата

$set 9 #Keys

1 Команди: Грешки во линијата
2 Команди: Не успеав да го спојам дрвото на команди!
3 Команди: Невалиден команда/менувач во линијата

$set 10 #Menu

1 Конфигурација
2 Fluxbox основно мени
3 Излез
4 Икони
5 Слој...
6 On Head...
7 Поставување
8 Превчитај конфигурација
9 Рестартирај
10 Предупредување: небалансирани [кодовни] тагови

$set 11 #Remember

1 Украси
2 Димензии
3 Скокни до Раб. место
4 Слој
5 Запамти...
6 Местоположба
7 Сними на затворање
8 Засенето
9 Лепливо
10 Непозната апликациска контрола
11 раб. место
12 Глава
13 Проѕирност

$set 12 #Screen

1 BScreen::BScreen: Друг управувач на прозорци е активен на сите екрани
2 W: %4d x H: %4d
3 BScreen::BScreen: управување на екранот %d со 0x%lx и длабочиња %d\n
4 Ш: %04d x В: %04d

$set 13 #Slit

1 Странки
2 Врти Надолу
3 Врти Нагоре
4 Слит Правец
5 Слит Слој
6 Слит на чело
7 Ориентација на Слит
8 Слит
9 Сними листа на Слитови

$set 14 #Toolbar

1 Реименувај го работното место
2 Iconbar Mode
3 Сите Прозорци
4 Икони
5 Ништо
6 Раб. место
7 Раб. место со Икони
8 Слој со Алатки
9 Toolbar on Head
10 Местопоположба на Алатник
11 Алатник
12 Ширина на Алатник
13 Часовник: 24h
14 Часовник: 12h
15 Уреди приказ на часовник
16 Прикажу икони
17 Без Икони
18 Раб. место без икони

$set 15 #Window

1 Неименуван

$set 16 #Windowmenu

1 Затвори
2 Скриј
3 Слој
4 Спушти
5 Зголеми
6 Крени
7 Прати до ...
8 Засени
9 Залепи
10 Уништи
11 Користи основно

$set 17 #Workspace

1 Работното место %d
2 Работни места
3 Ново Раб. место
4 Избриши последно

$set 18 #fbsetroot

1 Грешка: Мора да одберите: -solid, -mod, -gradient\n
2 Неможам да направам иконски атоми, се откажувам!
3 -display <текст>        прикажи поврзување\n\
-mod <x> <y>             модула шема\n\
-foreground, -fg <боја> модула боја на \n\
-background, -bg <боја> модула боја на позадина\n\n\
-gradient <состав>      приливен состав\n\
-from <боја>            почетна боја на прилив\n\
-to <боја>              крајна боја на прилив\n\n\
-solid <боја>           еднобојна\n\n\
-help                    се прикажува само овој текст и се исклучува програмата\n

$set 19 #main

1 грешка: '-display' бара аргумент
2 Грешно преместување
3 Грешно доделување
4 Надвор од домет
5 Runtime error
6 Вообичаен исклучок
7 Непозната грешка
8 грешка: '-log' бара аргумент
9 Записнички фајл
10 Најави се
11 грешка: '-rc' бара аргумент
12 грешка: '-screen' бара аргумент
13 Fluxbox %s: (c) %s Fluxbox Team\n\
Вебсајт: http://www.fluxbox.org/\n\n\
-display <string>\t\tuse display connection.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tuse alternate resource file.\n\
-version\t\t\tприкажува верзија на софтверот и се исклучува.\n\
-info\t\t\t\tприкажува корисни податоци.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tводи записник во фајл.\n\
-help\t\t\t\tја прикажува оваа текст помош и се исклучува .\n\n
14 предупредување: неможам да ја наместам околинската променлива 'DISPLAY'

$set 20 #layers

1 Над котва
2 Дно
3 Работна површина
4 Котва
5 Нормално
6 Над се
