$ codeset=BIG5

$set 1 #Align

1 下方居中
2 左下方
3 右下方
4 水平
5 靠左
6 左下方
7 左居中
8 左上方
9 展
10 靠右
11 右下方
12 右居中
13 右上方
14 上方居中
15 左上方
16 左上方
17 垂直

$set 2 #BaseDisplay

1 退出... 存崩文件\n
2 \n
3 %s:      捕信 %d\n

$set 3 #Common

1 Alpha值
2 自藏
3 
4 
5 器
6 器版本
7 	初始化
8 	
9 	本地言支持
10 	菜
11 	式
12 默配置
13 禁用
14 
15 Fluxbox版本
16 最大化
17 GIT修版本
18 可
19 主未定任何背景。\n考手或FAQ。

$set 4 #Configmenu

1 反
2 自提升
3 提升
4 聚焦
5 窗口
6 鼠切工作
7 Focus Window on Workspace Change
8 聚焦模式
9 聚焦新窗口
10 完全最大化
11 像仿色
12 移示窗口容
14 鼠聚焦
15 工作回旋
16 制透明
17 菜Alpha值
18 透明
19 聚焦窗口Alpha值
20 非聚焦窗口Alpha值
21 
22 置于
23 外置度
24 聚焦
25 鼠聚焦

$set 5 #Ewmh

1 致命: 存不足,法EWMH client list分配存

$set 6 #FbTkError

1 警告！ Height > 3200 ，置 Height = 3200
2 警告！ Width > 3200 ，置 Width = 3200
3 警告: 不能加用字体
4 分配。
5 建像素映射出
6 法建XImage
7 取失
8 警告：法alpha找到有效格式。
9 警告：建alpha片失。
10 警告：建alpha像素映射失。
11 查找屏幕格式失(%d)
12 存不足
13 存不足，法分配blue buffer。
14 存不足，法分配green buffer。
15 存不足，法分配red buffer。
16 取主目失
17 不支持像模式
18 置默值

$set 7 #Fluxbox

1 效的rc文件名!
2 加文件失
3 加据失
4 Retrying with
5 警告！法找到屏幕制窗口！
6 法建 %s 目
7 法找到需要控制的屏幕。\n确保有其他窗口管理器正在行。
8 法解析
9 致命！fluxbox只能有一例。
10 法接X server。\n确保你Fluxbox之前了X。
11 警告：X server不支持locale
12 警告：法置locale modifiers

$set 8 #Gnome

1 : 存不足, 法GNOME client list分配存

$set 9 #Keys

1 Keys: Error on line
2 Keys: Failed to merge keytree!
3 Keys: Invalid key/modifier on line

$set 10 #Menu

1 配置
2 Fluxbox默菜
3 退出
4 
5 次...
6 位于器...
7 位置
8 重新加配置文件
9 重Fluxbox
10 警告：[encoding]不

$set 11 #Remember

1 
2 尺寸
3 跳至工作
4 
5 住...
6 位置
7 保存
8 卷起
9 粘住
10 未知按
11 工作
12 器

$set 12 #Screen

1 BScreen::BScreen: X server生.\n 另外一窗口管理器正行,display
2 W: %4d x H: %4d
3 BScreen::BScreen: 控制屏幕 %d 使用像模式 0x%lx, 色彩深度 %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 Clients
2 下卷
3 上卷
4 Slit方向
5 Slit次
6 Slit位于器
7 Slit位置
8 Slit
9 保存Slit列表

$set 14 #Toolbar

1 重命名前工作
2 模式
3 所有窗口
4 所有最小化的窗口
5 
6 前工作所有窗口
7 前工作所有最小化的窗口
8 工具次
9 工具位于器
10 工具位置
11 工具
12 工具度百分比
13 ：24小格式
14 ：12小格式
15 示格式
16 示
17 所有未最小化的窗口
18 前工作所有未最小化的窗口

$set 15 #Window

1 未命名

$set 16 #Windowmenu

1 
2 最小化
3 次
4 降低
5 最大化
6 提升
7 送到...
8 卷起
9 粘住
10 死

$set 17 #Workspace

1 工作 %d
2 工作
3 新建工作
4 移除最后一工作

$set 18 #fbsetroot

1 : 必指定: -solid, -mod, -gradient 三中的一.\n
2 法建pixmap atoms，放！
3 -display <string>        display connection\n\
-mod <x> <y>             modula pattern\n\
-foreground, -fg <color> modula foreground color\n\
-background, -bg <color> modula background color\n\n\
-gradient <texture>      gradient texture\n\
-from <color>            gradient start color\n\
-to <color>              gradient end color\n\n\
-solid <color>           solid color\n\n\
-help                    示助信息\n

$set 19 #main

1 : '-display'需要一
2 Bad Alloc
3 Bad cast
4 超出示范
5 行
6 准例外
7 未知
8 : '-log'需要一
9 日志文件
10 日志到
11 : '-rc'需要一
12 : '-screen'需要一
13 Fluxbox %s: (c) %s Fluxbox Team\n\
主: http://www.fluxbox.org/\n\n\
-display <string>\t\t指定display接。\n\
-screen <all|int,int,int>\t在特定的屏幕上示。\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\t使用指定的源文件。\n\
-version\t\t\t示版本然后退出。\n\
-info\t\t\t\t示一些有用的信息。\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\t出到文件中。\n\
-help\t\t\t\t示助信息然后退出。\n\n
14 警告: 法置境量'DISPLAY'

$set 20

1 在Dock之上
2 底部
3 桌面
4 Dock
5 普通
6 部


