$ codeset=ISO-8859-9

$set 1 #Align

1 Alt Orta
2 Alt Sol
3 Alt Sağ
4 Yatay
5 Sol
6 Sol Alt
7 Sol Orta
8 Sol Üst
9 Bağlı
10 Sağ
11 Sağ Alt
12 Sağ Orta
13 Sağ üst
14 Üst Orta
15 Üst Sol
16 Üst Sağ
17 Dikey
18 Orta
19 Üst
20 Alt

$set 2 #BaseDisplay

1 İptal Ediliyor... çekirdek dökümü alınıyor\n
2 Kapatılıyor\n
3 %s:      %d sinyali alındı\n

$set 3 #Common

1 Alfa
2 Otomatik Gizle
3 Derlenmiş
4 Derlenmiş ayarlar
5 Derleyici
6 Derleyici versiyonu
7      başlat
8      tuşlar
9         nls
10        menü
20  windowmenu
11        stil
12 Varsayılanlar
13 devre dışı
14 Hata
15 Fluxbox sürümü
16 En Üste Al
17 GIT Revizyonu
18 Görünür
19 Bu biçemde tanımlanmış arkaplan seçeneği yok.\nLütfen kılavuzu ya da SSS'yi okuyun.

$set 4 #Configmenu

1 Kenar Düzleştirme
2 Otomatik Büyütme
3 Tıklayarak Büyütme
4 Odaklamak için Tıkla
5 Geçici Pencereleri Süsle
6 Masaüstleri Arası Fare Tekerleği ile Geçiş
7 Çalışma Alanı Değiştiğinde Pencereye Odaklan
8 Kalıba Odakla
9 Yeni Pencereye Odakla
10 Tamamen Büyütme
11 Resim Titretme
12 Opak Pencere Taşıma
14 Fare Odağı
15 Çalışma Alanı Sapması
16 Sahte Saydamlık için Zorla
17 Menü Alfası
18 Saydamlık
19 Odaklanılmış Pencere Alfası
20 Odaklanılmamış Pencere Alfası
21 Sekme Özellikleri
22 Başlık Çubuğunda Sekmeler
23 Harici Sekme Genişliği
24 Tıklama Sekmesi Odağı
25 Fare Sekmesi Odağı
26 Ayarları En Üst Düzeye Getir
27 Boyutlandırma Arttırmasını Görmezden Gel
28 Taşımayı Devre Dışı Bırak
29 Boyutlandırmayı Devre Dışı Bırak

$set 5 #Ewmh

1 Ölümcül: Bellek Yetersiz, EWMH istemci listesi için ayrılamıyor

$set 6 #FbTkError

1 Uyarı! Boy > 3200, Boy 3200 olarak ayarlanıyor
2 Uyarı! En > 3200, En 3200 olarak ayarlanıyor
3 Uyarı! ikincil yazı tipi yüklenemiyor
4 Ayırma hatası.
5 Pixmap oluştururken Hata
6 XImage oluşturulamıyor
7 Okunamıyor
8 Uyarı: Alfa için uygun biçim bulunamadı.
9 Uyarı: Alfa resmi oluşturulamadı.
10 Uyarı: Alfa pixmap'ı oluşturulamadı.
11 Ekran(%d) için biçim bulunamadı
12 Bellek yetersiz
13 Mavi tampon için bellek ayrılırken bellek yetersiz.
14 Yeşil tampon için bellek yarılırken bellek yetersiz.
15 Kırmızı tampon için bellek ayrılırken bellek yetersiz.
16 Tema ögesi okunamadı
17 desteklenmeyen görsel
18 Varsayılan değer ayarlanıyor

$set 7 #Fluxbox

1 rc dosya adı geçersiz!
2 Groupfile yüklenemedi
3 Veritabanı yüklenemedi
4 Tekrar deneniyor:
5 Uyarı! Pencereleri eşleyecek ekran bulunamıyor!
6 %s dizini oluşturulamıyor
7 Düzenlenecek pencereler bulunamıyor.\nBaşka bir pencere yöneticisinin çalışmadığından emin olun.
8 Düzenli ifadeler ayrıştırılırken hata
9 Ölümcül! Fluxbox sınıflarından sadece bir örnek olabilir.
10 X sunucusuna bağlanılamıyor.\nFluxbox'u başlatmadan önce X'i başlattığınızdan emin olun.
11 Uyarı: X sunucusu yerelleştirmeyi desteklemiyor
12 Uyarı: Yerelleştirme tanımlayıcıları ayarlanamıyor

$set 8 #Gnome

1 Ölümcül: Bellek yetersiz, GNOME istemci listesi için yer ayrılamıyor

$set 9 #Keys

1 Tuşlar: Satırda hata
2 Tuşlar: Tuş ağacı birleştirilirken hata!
3 Tuşlar: Satırda geçersiz tuş/tanımlayıcı

$set 10 #Menu

1 Yapılandırma
2 Fluxbox varsayılan menü
3 Çıkış
4 Simgeler
5 Katman...
6 En Başta...
7 Yerleştirme
8 Yapılandırmayı Yeniden Yükle
9 Yeniden başlat
10 Uyarı: dengesiz [şifreleme] etiketleri

$set 11 #Remember

1 Süslemeler
2 Boyutlar
3 Çalışma alanına atla
4 Katman
5 Hatırla...
6 Konum
7 Çıkışta kaydet
8 Gölgelenmiş
9 Yapışkan
10 Bilinmeyen uygulama tuşu
11 Çalışma Alanı
12 Baş
13 Saydamlık

$set 12 #Screen

1 BScreen::BScreen: X sunucusu sorgulanırken hata oluştu.\n	şu an ekranda başka bir pencere yöneticisi çalışıyor
2 E: %4d x B: %4d
3 BScreen::BScreen: %d ekranı düzenleniyor 0x%lx görseli, %d derinliği kullanılıyor\n
4 E: %04d x B: %04d

$set 13 #Slit

1 İstemciler
2 Devir aşağı
3 Devir yukarı
4 Kesit Yönü:
5 Kesit Katmanı
6 Kesit Tepede
7 Kesit Yerleşimi
8 Kesit
9 Kesit Listesini kaydet

$set 14 #Toolbar

1 Geçerli çalışma alanını adlandır
2 Simge Çubuğu Kipi
3 Tüm Pencereler
4 Simgeler
5 Hiç
6 Çalışma Alanı
7 Çalışma Alanı Simgeleri
8 Araç Çubuğu Katmanı
9 Araç Çubuğu En Üstte
10 Araç Çubuğu Yerleştirmesi
11 Araç Çubuğu
12 Araç çubuğu genişlik yüzdesi
13 Saat: 24s
14 Saat: 12s
15 Saat Biçimini Düzenle
16 Resimleri Göster
17 SimgelerYok
18 ÇalışmaAlanıSimgelerYok

$set 15 #Window

1 İsimsiz

$set 16 #Windowmenu

1 Kapat
2 Simge boyutuna küçült
3 Katman
4 Düşür
5 Büyüt
6 Yükselt
7 Şuraya Gönder ...
8 Gölge
9 Yapış
10 Öldür
11 Varsayılanları Kullan
12 Pencere Başlığını Ayarla

$set 17 #Workspace

1 Çalışma Alanı %d
2 Çalışma Alanları
3 Yeni Çalışma Alanı
4 Sonuncuyu Sil

$set 18 #fbsetroot

1 Hata: Bunlardan en az birini belirtmelisiniz: -solid, -mod, -gradient\n
2 pixmap atomları oluşturulamıyor, bırakılıyor!
3 -display <string>        bağlantıyı görüntüle\n\
-mod <x> <y>             mod kalıbı\n\
-foreground, -fg <color> mod önplan rengi\n\
-background, -bg <color> mod arkaplan rengi\n\n\
-gradient <texture>      doku eğimi\n\
-from <color>            başlangıç renk eğimi\n\
-to <color>              bitiş renk eğimi\n\n\
-solid <color>           rengi katılaştırır\n\n\
-help                    bu yardım ekranını görüntüle ve çık\n

$set 19 #main

1 hata: '-display' bir parametre ile birlikte kullanılıyor
2 Kötü Ayırma
3 Kötü biçim
4 Aralık dışı
5 Çalışma zamanı hatası
6 Standart İstisna
7 Bilinmeyen hata
8 hata: '-log' bir parametre ile birlikte kullanılıyor
9 Günlük Dosyası
10 Günlük tutuluyor: 
11 hata: '-rc' bir parametre ile birlikte kullanılıyor
12 hata: '-screen' bir parametre ile birlikte kullanılıyor
13 Fluxbox %s: (c) %s Fluxbox Takımı\n\
Web sitesi: http://www.fluxbox.org/\n\n\
-display <string>\t\tekran bağlantısını kullan.\n\
-screen <all|int,int,int>\tsadece belirlenen ekranlarda çalış.\n\
-no-slit\t\t\tbir kesit sağlama.\n\
-no-toolbar\t\t\tbir araç çubuğu sağlama.\n\
-rc <string>\t\t\tulternatif kaynak dosyasını kullan.\n\
-version\t\t\tsürümü görüntüle ve çık.\n\
-info\t\t\t\tyararlı bilgi görüntüle.\n\
-list-commands\t\t\tbütün geçerli tuş komutlarını listele.\n\
-sync\t\t\t\thata ayıklama için X sunucusu ile eşzamanla.\n\
-log <filename>\t\t\tgünlük çıktısını dosyaya yaz.\n\
-help\t\t\t\tbu yardım ekranını göster ve çık.\n\n
14 uyarı: çevre değişkeni 'DISPLAY' ayarlanamıyor

$set 20 #layers

1 Panelin Üstü
2 Alt
3 Masaüstü
4 Panel
5 Normal
6 Üst

