$ codeset=UTF-8

$set 1 #Align

1 תחת מרכזי
2 תחת שמאלי
3 תחת ימני
4 מאוזן
5 שמאל
6 שמאל תחתון
7 שמאל מרכז
8 שמאל עליון
9 יחסי
10 ימין
11 ימין תחתון
12 ימין מרכז
13 ימין עליון
14 שיא מרכזי
15 שיא שמאלי
16 שיא ימני
17 מאונך
18 מרכז
19 שיא
20 תחת

$set 2 #BaseDisplay

1 Aborting... dumping core\n
2 Shutting Down\n
3 %s:      signal %d caught\n

$set 3 #Common

1 אלפא
2 הסתר אוטומטית
3 Compiled
4 Compiled options
5 Compiler
6 Compiler version
7        init
8        keys
9         nls
10        menu
20  windowmenu
11       style
12 Defaults
13 disabled
14 Error
15 Fluxbox version
16 הגדל על פני
17 GIT Revision
18 נראה
19 There is no background option specified in this style.\nPlease consult the manual or read the FAQ.

$set 4 #Configmenu

1 AntiAlias_OBSOLETE
2 הרם אוטומטית
3 קליק מרים
4 קליק למקד
5 עטר חלונות ארעים
6 Desktop MouseWheel Switching_OBSOLETE
7 מקד חלון בעת שינוי מרחב עבודה
8 אימום מיקוד
9 מקד חלונות חדשים
10 הגדלה מרבית
11 מיזוג צבע צלם
12 הזזת חלון אטומה
13 מיקוד עכבר (מוחלט)
14 מיקוד עכבר
15 כריכת שולחן עבודה
16 כפה שקיפות-פסבדו
17 אלפא של תפריט
18 שקיפות
19 אלפא של חלון ממוקד
20 אלפא של חלון לא ממוקד
21 אפשרויות כרטיסייה
22 כרטיסיות בתוך שורת כותרת
23 רוחב כרטיסייה חיצונית
24 מיקוד כרטיסייה קליק
25 מיקוד כרטיסייה עכבר
26 אפשרויות הגדלה
27 התעלם מהגדלת מידה
28 נטרל הזזה
29 נטרל שינוי מידה

$set 5 #Ewmh_OBSOLETE

1 Fatal: Out of memory, can't allocate for EWMH client list_OBSOLETE

$set 6 #FbTkError

1 Warning! Height > 3200, setting Height = 3200
2 Warning! Width > 3200, setting Width = 3200
3 Warning: can't load fallback font
4 Allocation error.
5 Error creating pixmap
6 Can't create XImage
7 Failed to read
8 Warning: Failed to find valid format for alpha.
9 Warning: Failed to create alpha picture.
10 Warning: Failed to create alpha pixmap.
11 Failed to find format for screen(%d)
12 Out of memory
13 Out of memory while allocating blue buffer.
14 Out of memory while allocating green buffer.
15 Out of memory while allocating red buffer.
16 Failed to read theme item
17 unsupported visual
18 Setting default value

$set 7 #Fluxbox

1 rc filename is invalid!
2 Failed to load groupfile_OBSOLETE
3 Failed to load database
4 Retrying with
5 Warning! Could not find screen to map window on!
6 Can't create directory %s
7 Couldn't find screens to manage.\nMake sure you don't have another window manager running.
8 Error parsing regular expression
9 Fatal! There can only one instance of fluxbox class.
10 Can not connect to X server.\nMake sure you started X before you start Fluxbox.
11 Warning: X server does not support locale
12 Warning: cannot set locale modifiers

$set 8 #Gnome_OBSOLETE

1 Fatal: Out of memory, can't allocate for GNOME client list_OBSOLETE

$set 9 #Keys

1 Keys: Error on line
2 Keys: Failed to merge keytree!_OBSOLETE
3 Keys: Invalid key/modifier on line

$set 10 #Menu

1 תצורה
2 תפריט משתמט Fluxbox
3 יציאה
4 צלמיות
5 רובד...
6 על ראש...
7 מיקום
8 טען תצורה מחדש
9 ריסט
10 Warning: unbalanced [encoding] tags

$set 11 #Remember

1 עיטורים
2 ממדים
3 קפוץ אל מרחב עבודה
4 רובד
5 זכור...
6 עמדה
7 שמור בעת סגירה
8 מואפל
9 דביק
10 Unknown apps key_OBSOLETE
11 מרחב עבודה
12 ראש
13 שקיפות
14 ממוזער
15 מוגדל
16 מסך מלא

$set 12 #Screen

1 BScreen::BScreen: an error occured while querying the X server.\n	another window manager already running on display
2 W: %4d x H: %4d
3 BScreen::BScreen: managing screen %d using visual 0x%lx, depth %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 לקוחות
2 העבר מטה
3 העבר מעלה
4 כיוון סדק:
5 רובד סדק
6 סדק על ראש
7 מיקום סדק
8 סדק
9 שמור רשימה

$set 14 #Toolbar

1 שנה שם מרחב עבודה נוכחי
2 מצב סרגל צלמיות
3 כל החלונות
4 צלמיות
5 כלום
6 מרחב עבודה
7 מרחב עבודה צלמיות
8 רובד סרגל כלים
9 סרגל כלים על ראש
10 מיקום סרגל כלים
11 סרגל כלים
12 אחוז רוחב סרגל כלים
13 שעון: 24ש
14 שעון: 12ש
15 ערוך פורמט שעון
16 הצג תמונות
17 אין צלמיות
18 מרחב עבודה אין צלמיות

$set 15 #Window

1 ללא שם

$set 16 #Windowmenu

1 סגור
2 הקטן
3 רובד
4 הנמך
5 הגדל
6 הרם
7 שלח אל ...
8 האפל
9 דביק
10 הרוג
11 השתמש בברירות מחדל
12 קבע כותרת חלון

$set 17 #Workspace

1 מרחב עבודה %d
2 מרחבי עבודה
3 מרחב עבודה חדש
4 הסר אחרון

$set 18 #fbsetroot

1 Error: must specify one of: -solid, -mod, -gradient\n
2 Couldn't create pixmap atoms, giving up!
3 -display <string>        display connection\n\
-mod <x> <y>             modula pattern\n\
-foreground, -fg <color> modula foreground color\n\
-background, -bg <color> modula background color\n\n\
-gradient <texture>      gradient texture\n\
-from <color>            gradient start color\n\
-to <color>              gradient end color\n\n\
-solid <color>           solid color\n\n\
-help                    print this help text and exit\n

$set 19 #main

1 error: '-display' requires an argument
2 Bad Alloc
3 Bad cast
4 Out of range
5 Runtime error
6 Standard Exception
7 Unknown error
8 error: '-log' needs an argument
9 Log File
10 Logging to
11 error: '-rc' requires an argument
12 error: '-screen' requires an argument
13 Fluxbox %s: (c) %s Fluxbox Team\n\
Website: http://www.fluxbox.org/\n\n\
-display <string>\t\tuse display connection.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\t.do not provide a toolbar.\n\
-rc <string>\t\t\tuse alternate resource file.\n\
-version\t\t\tdisplay version and exit.\n\
-info\t\t\t\tdisplay some useful information.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tlog output to file.\n\
-help\t\t\t\tdisplay this help text and exit.\n\n
14 warning: couldn't set environment variable 'DISPLAY'

$set 20 #layers

1 מעל רציף
2 תחתון
3 שולחן עבודה
4 רציף
5 רגיל
6 עליון


