$ codeset=ISO-8859-1

$set 1 #Align

1 Nede i midten
2 Nede til venstre
3 Nede til høyre
4 Horisontelt
5 Venstre
6 Til venstre nede
7 Til venstre i midten
8 Til venstre opp
9 Relativt
10 Høyre
11 Til høyre nede
12 Til høyre i midten
13 Til høyre oppe
14 Oppe i midten
15 Oppe til venstre
16 Oppe til høyre
17 Vertikalt

$set 2 #BaseDisplay

1 Avbryter.. dumper kjernen\n
2 Avslutter\n
3 %s:    signal %d fanget\n

$set 3 #Common

1 Gjenomsiktlighet
2 Gjem automatisk
3 Kompilert
4 Kompilerte valg
5 Kompilator
6 Kompilator valg
7 init-fil
8 Taste-fil
9 Språk
10 Meny-fil
11 Stil
12 Standard valg
13 ikke aktivert
14 Valg
15 Fluxbox versjon
16 Forstørr over
17 GIT versjon
18 Synlig


$set 4 #Configmenu

1 Kantutjevning
2 Vis automatisk
3 Klikk hever
4 Klikk for fokus
5 Dekorer gjennomsiktige vinduer
6 Skrivebord Mushjul Bytting
7 Fokuser vindu ved skrivebordsbytte
8 Fokusmodell
9 Fokus på nye vinduer
10 Full maximering
11 Bildedithering
12 Ugjennomsiktelig vinduflyttning
13 Halvsløv fokus
14 Sløv fokus
15 Dra vindu mellon arbeidsplasser
16 Tving kunstig gjennomsiktighet
17 Meny gjennomsiktighet
18 Gjennomsiktighet
19 Fokusert vindugjennomsiktighet
20 Ufokusert vindugjennomsiktighet

$set 5 #Ewmh

1 Farlig: Tom for minne, kan ikke allokere for EWHM klient listen

$set 6 #FbTkError

1 Advarsel! Høyden > 3200 setter Høyde = 3200
2 Advarsel! Vidden > 3200 setter Vidde = 3200
3 Advarsel! Kan ikke finne font
4 Allokerings feil
5 Feil ved opprettelse av pixmap
6 Kan ikke opprette XImage
7 Klarte ikke å lese

$set 7 #Fluxbox

1 rc filnavn er ikke gyldig!
2 Klarte ikke laste gruppefil
3 Klarte ikke laste gruppefil
4 Prøver igjen med
5 Advarsel! Kunne ikke finne skjerm å skrive til
6 Kan ikke opprette mappen %s

$set 8 #Gnome


$set 9 #Keys


$set 10 #Menu

1 Konfigurasjon
2 Fluxbox standard meny
3 Avslutt
4 Ikoner
5 Lag...
7 Plassering
8 Last konfig på ny
9 Start på ny

$set 11 #Remember

1 Dekorasjoner
2 Dimensjoner
3 Hopp til arbeidsplass
4 Lag
5 Husk...
6 Posisjon
7 Lagre ved lukking
8 Skygge
9 Klister

$set 12 #Screen

2 W: %4d x H: %4d
4 W: %04d x H: %04d

$set 13 #Slit

4 Slitretning
7 Slitplassering
8 Slit

$set 14 #Toolbar

1 Endre skrivebordsnavn
2 Ikonlinje modus
3 Alle vinduer
4 Ikoner
5 Ingen
6 Arbeidsplass
7 Arbedisplass ikoner
8 Verktøylinje lag
10 Verktøylistens plassering
11 Verktøysliste
12 Verktøyliste vidde i prosent
13 Klokke: 24t
14 Klokke: 12t
15 Endre klokkeformat
16 Vis bilder
17 Ingen ikoner

$set 15 #Window

1 Intet navn

$set 16 #Windowmenu

1 Lukk
2 Ikonifiser
3 Lag
4 Senk
5 Maksimer
6 Høyne
7 Send til...
8 Skygge
9 Klebrig
10 Drep

$set 17 #Workspace

1 Skrivebord %d
2 Skrivebord
3 Nytt skrivebord
4 Fjern siste

$set 18 #fbsetroot

1 fel: måste specifiera en av: -solid, -mod, -gradient\n
3 -display <string>        skärmanslutning\n\
-mod <x> <y>             modulamönster\n\
-foreground, -fg <color> förgrundsfärg för modulamönster\n\
-background, -bg <color> bakgrundsfärg för modulamönster\n\n\
-gradient <texture>      texturlutning\n\
-from <color>            startfärg\n\
-to <color>              slutfärg\n\n\
-solid <color>           solid färg\n\n\
-help                    visa denna hjälptext och avsluta\n

$set 19 #main

1 feil: '-display' krever ett argument
11 fel: '-rc' krever ett argument
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tanvend skjermanslutning.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <sträng>\t\t\tanvänd alternativ resursfil.\n\
-version\t\t\tvisa version och avsluta.\n\
-info\t\t\t\tVisar nyttig information.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filenamn>\t\t\tloggar utskriften till en fil.\n\
-help\t\t\t\tvisa denna hjälptext och avsluta.\n\n


