$ codeset=UTF-8

$set 1 #Align

1 Phía dưới ở giữa
2 Phía dưới bên trái
3 Phía dưới bên trái
4 Nằm ngang
6 Bên trái phía dưới
7 Bên trái ở giữa
8 Bên trái ở trên
11 Bên phải ở dưới
12 Bên phải ở giữa
13 Bên phải ở trên
14 Bên trên ở giữa
15 Bên trên phía trái
16 Bên trên phía phải
17 Thẳng đứng

$set 2 #BaseDisplay

1 Dừng...dumping core\n
2 Tắt máy\n
3 %s:		bắt được tín hiệu %d\n

$set 3 #Common

1 Độ trong suốt
2 Tự động ẩn
3 Được biên dịch
4 Tùy chọn được biên dịch
5 Trình biên dịch
6 Phiên bản trình biên dịch
7 Xác lập cơ bản
8 Phím nóng
10 Thực đơn
11 Sắc thái
12 Mặc định
13 Không chọn
14 Lỗi
15 Phiên bản fluxbox
16 Phóng đại che hết màn hình
18 Có thể nhìn thấy

$set 4 #Configmenu

4 Tiêu điểm theo nhấn chuột
7 Làm tiêu điểm cửa sổ cuối cùng
8 Mẫu tiêu điểm
9 Đưa tiêu điểm lên cửa sổ mới
10 Phóng đại hết cỡ
11 Làm phẳng hình ảnh
12 Hiện nội dung cửa sổ khi di chuyển 
14 Tiêu điểm theo trỏ chuột
15 Di chuyển cửa sổ giữa các không gian làm việc

$set 5 #Ewmh

1 Lỗi: Không đủ bộ nhớ để phân phối cho danh sách client EWMH.

$set 6 #FbTkError

1 Cảnh báo! Chiều cao > 3200 đặt chiều cao = 3200
2 Cảnh báo! Chiều rộng > 3200 đặt chiều rộng = 3200
3 Cảnh báo: không thể nạp phông chữ fallback
4 Lỗi phân phối
5 Lỗi tạo pixmap
6 Không thể tạo XImage
7 Lỗi đọc
8 Cảnh báo: lỗi định dạng của độ trong suốt
9 Cảnh báo: lỗi tạo ảnh trong suốt
10 Cảnh báo: lỗi tạo pixmap trong suốt
11 Lỗi định dạng cho màn hình (%d)
12 Đầy bộ nhớ
13 Đầy bộ nhớ khi phân phối cho bộ đệm Blue
14 Đầy bộ nhớ khi phân phối cho bộ đệm Green
15 Đầy bộ nhớ khi phân phối cho bộ đệm Red
16 Lỗi đọc tệp tin sắc thái
17 Tùy chọn không được hỗ trợ
18 Thiết lập giá trị mặc định

$set 7 #Fluxbox

1 Lỗi tên tệp tin rc!
2 Lỗi nạp tệp tin nhóm
3 Lỗi nạp cơ sở dữ liệu
4 Thử lại với
5 Cảnh báo! Không tìm thấy màn hình để hiển thị cửa sổ
6 Không thể tạo thư mục %s
7 Không thể tìm thấy màn hình để quản lý.\nHãy kiểm tra xem bạn có đang chạy một trình quản lý màn hình khác không.
8 Lỗi phân tích biểu thức chính quy
9 Lỗi! Chỉ cho phép một trường hợp của fluxbox class.
10 Không thể kết nối với máy chủ X.\nCần chạy X trước khi chạy fluxbox.
11 Cảnh báo: máy chủ X không hỗ trợ bản dịch địa phương
12 Cảnh báo: không thể thiết lập bản dịch địa phương

$set 8 #Gnome

1 Lỗi: không đủ bộ nhớ để phân phối cho danh sách client GNOME

$set 9 #Keys

1 Phím nóng: Lỗi trên một dòng
2 Phím nóng: Lỗi kết hợp cây phím!
3 Phím nóng: Lỗi phím/chức năng

$set 10 #Menu

1 Cấu hình
2 Thực đơn mặc định
3 Thoát
4 Cửa sổ thu nhỏ
5 Lớp
7 Vị trí
8 Nạp lại cấu hình
9 Khởi động lại

$set 11 #Remember

1 Trang trí
2 Kích thước
3 Nhảy tới không gian
4 Lớp
5 Nhớ...
6 Vị trí
7 Ghi nhớ khi đóng
8 Thu nhỏ vào tiêu đề
9 Hiện trên mọn không gian
10 Phím chưa được xác định
11 Không gian

$set 12 #Screen

2 Rộng: %4d x Cao: %4d
4 Rộng: %04d x Cao: %04d

$set 13 #Slit

1 Các client
2 Quay vòng xuống
3 Quay vòng lên
4 Hướng của Slit
5 Lớp của Slit
6 Slit trên Head
7 Vị trí của Slit
8 Slit

$set 14 #Toolbar

1 Thay tên không gian hiện thời
2 Chế độ thanh cửa sổ thu nhỏ
3 Tất cả cửa sổ
4 Biểu tượng
5 Không dùng
6 Không gian
7 Biểu tượng không gian
8 Lớp thanh tác vụ
9 Thanh tác vụ trên Head
10 Vị trí thanh tác vụ
11 Thanh tác vụ
12 Chiều rộng thanh tác vụ

$set 15 #Window

1 Không có tên

$set 16 #Windowmenu

1 Đóng
2 Thu nhỏ xuống thanh tác vụ
4 Lên nền sau
5 Phóng đại
6 Lên nền trước
7 Chuyển đến ...
8 Thu nhỏ vào tiêu đề
9 Trên mọi không gian

$set 17 #Workspace

1 Không gian %d
2 Không gian
3 Không gian mới
4 Xóa không gian cuối

$set 18 #fbsetroot

1 lỗi: cần đưa ra một trong các từ khóa sau: -solid, -mod, -gradient\n
2 Không thể tạo các pixmap!
3 -display <string>        hiển thị trên màn hình <tên màn hình>\n\
-mod <x> <y>             mẫu môđun <x> <y>\n\
-foreground, -fg <color> màu nền trước <màu>\n\
-background, -bg <color> màu nền sau <màu>\n\n\
-gradient <texture>      dải màu <kết cấu>\n\
-from <color>            từ <màu>\n\
-to <color>              đến <màu>\n\n\
-solid <color>           màu thuần nhất <màu>\n\n\
-help                    hiển thị thông tin trợ giúp này và thoát\n

$set 19 #main

1 lỗi: '-display' cần đối số
2 Phân phối không tốt
3 Chia sẻ không tốt
4 Lỗi ngoài hàng
5 Lỗi runtime
6 Sự loại trừ tiêu chuẩn
7 Lỗi không rõ nguyên nhân
8 lỗi: '-log' cần một đối số <tên tệp tin>
9 Tệp tin sự kiện
10 Ghi sự kiện vào
11 lỗi: '-rc' cần đối số
13 Fluxbox %s: (c) %s Fluxbox Team\n
Website: http://www.fluxbox.org/\n\n\
-display <string>\t\tsử dụng màn hình lựa chọn.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tsử dụng tệp tin tài nguyên khác.\n\
-version\t\t\tđưa ra số phiên bản và thoát.\n\
-info\t\t\t\thiển thị vài thông tin có ích.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\ttên tệp tin sự kiện.\n\
-help\t\t\t\thiển thị thông tin trợ giúp này và thoát.\n\n


