$ codeset=KOI8-R

$set 1 #Align

1 Внизу по центру
2 Внизу слева
3 Внизу справа
4 По горизонтали
5 Слева
6 Слева внизу
7 Слева по центру
8 Слева вверху
9 По всей ширине
10 Справа
11 Справа внизу
12 Справа по центру
13 Справа вверху
14 Вверху по центру
15 Вверху слева
16 Вверху справа
17 По вертикали

$set 2 #BaseDisplay

1 Прервано... создан дамп памяти\n
2 Завершение работы\n
3 %s: сигнал %d получен\n

$set 3 #Common

1 Прозрачность
2 Автоскрытие
3 Скомпилированный
4 Скомпилирован с опциями
5 Компилятор
6 Версия компилятора
7 Файл ресурсов
8 Файл сочетаний клавиш
9 Файлы локализации
10 Файл меню
11 Файл со стилем
12 Настройки по умолчанию
13 отключена
14 Ошибка
15 Версия Fluxbox
16 Под окнами
17 Номер ревизии
18 Видимая
19 Для этого стиля не определена опция 'background'\nПодробную информацию смотрите в man fluxbox или в FAQ.

$set 4 #Configmenu

1 Антиалиасинг
2 Автовсплытие
3 На передний план по щелчку
4 Фокус по щелчку
5 Украшать нерезидентные окна
6 Переключение рабочих столов колесом мыши
7 Фокус на последнее окно рабочего стола
8 Модель фокусировки
9 Перемещать фокус на новые окна
10 Полная максимизация
11 Сглаживать изображение
12 Перемещение заполненных окон
14 Фокус по перемещению
15 Перемещение окон между рабочими столами
16 Использовать псевдо-прозрачность
17 Меню
18 Прозрачность
19 Активное окно
20 Неактивное окно
21 Вкладки
22 Размещать внутри заголовка окна
23 Ширина внешних вкладок
24 Переключение по вкладкам с помощью клика
25 Переключение по вкладкам при наведении мыши

$set 5 #Ewmh

1 Фатальная ошибка: не удалось выделить память для списка клиентов EWMH

$set 6 #FbTkError

1 Внимание! Высота > 3200, установлена Высота = 3200
2 Внимание! Ширина > 3200, установлена Ширина = 3200
3 Внимание: не удалось загрузить резервный шрифт
4 Ошибка выделения памяти.
5 Ошибка при создании pixmap
6 Невозможно создать XImage
7 Ошибка чтения
8 Внимание: Неверное значение alpha.
9 Внимание: Не удалось создать alpha picture.
10 Внимание: Не удалось создать alpha pixmap.
11 Не удалось найти формат экрана(%d)
12 Нехватка памяти
13 Нехватка памяти при выделении буфера синего.
14 Нехватка памяти при выделении буфера зеленого.
15 Нехватка памяти при выделении буфера красного.
16 Ошибка чтения элемента стиля
17 Неподдерживаемый visual 
18 Установлено значение по умолчанию

$set 7 #Fluxbox

1 Неправильное имя файла ресурсов
2 Невозможно загрузить файл групп
3 Невозможно загрузить базу
4 Повторная попытка с
5 Внимание! Не найден экран для отображения окна!
6 Невозможно создать директорию %s
7 Не удалось найти экран.\nУбедитесь, что не запущен другой WM.
8 Ошибка при разборе регулярного выражения
9 Фатальная ошибка! Экземпляр класса fluxbox может быть только один. 
10 Невозможно соединиться с X сервером.\nУбедитесь, что X сервер запущен перед запуском Fluxbox.
11 Внимание: X сервер не поддерживает локаль
12 Внимание: невозможно установить модификаторы локали

$set 8 #Gnome

1 Фатальная ошибка: не удалось выделить память для списка клиентов GNOME

$set 9 #Keys

1 Keys: Ошибка в строке 
2 Keys: Не удалось создать дерево ключей
3 Keys: Неверная клавиша/модификатор в строке

$set 10 #Menu

1 Настройки
2 Меню fluxbox
3 Выход
4 Свернутые окна
5 Слой...
6 В заголовок...
7 Расположение
8 Перечитать настройки
9 Перезапуск
10 Внимание: не закрыты тэги [encoding]

$set 11 #Remember

1 Декорации
2 Размеры
3 Переместиться на рабочий стол
4 Слой
5 Запомнить...
6 Расположение
7 Сохранять при закрытии
8 Свернуто в заголовок
9 Приклеенный
11 Рабочий стол
12 Заголовок
13 Прозрачность

$set 12 #Screen

1 BScreen::BScreen: произошла ошибка во время опроса X сервера.\n   запущен другой менеджер окон
2 W: %4d x H: %4d
3 BScreen::BScreen: управляем экраном %d используя visual 0x%lx, depth %d\n
4 W: %04d x H: %04d

$set 13 #Slit
1 Клиенты
2 Обычный порядок
3 В обратном порядке
4 Ориентация докера
7 Расположение докера
8 Докер
9 Сохранить список клиентов

$set 14 #Toolbar

1 Редактировать имя текущего рабочего стола
2 Режим отображения окон
3 Все окна со всех рабочих столов
4 Минимизированные на всех рабочих столах
5 Не отображать
6 Все окна
7 Минимизированные
10 Расположение панели инструментов
11 Панель инструментов
12 Ширина в процентах
13 Формат времени: 24 часа
14 Формат времени: 12 часов
15 Изменить формат отображения времени
16 Отображать иконки
17 Все не свернутые со всех рабочих столов
18 Не свернутые

$set 15 #Window

1 Безымянное

$set 16 #Windowmenu

1 Закрыть
2 Свернуть на панель инструментов
3 Слой...
4 На задний план
5 Максимизировать
6 На передний план
7 Отправить окно на ...
8 Свернуть в заголовок
9 Приклеить
10 Убить
11 Использовать установки по умолчанию

$set 17 #Workspace

1 Рабочий стол %d
2 Рабочие столы
3 Новый рабочий стол
4 Удалить последний

$set 18 #fbsetroot

1 Ошибка: необходимо задать один из следующих ключей: -solid, -mod, -gradient\n
2 Не удалось создать атомы pixmap, бросаем это гиблое дело!
3 -display <string>        соединение с дисплеем\n\
-mod <x> <y>             макет клетки\n\
-foreground, -fg <color> цвет переднего плана клетки\n\
-background, -bg <color> цвет фона клетки\n\n\
-gradient <texture>      градиент\n\
-from <color>            начальный цвет градиента\n\
-to <color>              конечный цвет градиента\n\n\
-solid <color>           сплошной цвет\n\n\
-help                    вывести эту подсказку и выйти\n

$set 19 #main

1 ошибка: '-display' требует наличие аргумента
2 Ошибка выделения памяти
3 Ошибка преобразования типа
4 Вне диапазона
5 Ошибка во время выполнения
6 Стандартное исключение
7 Неизвестная ошибка
8 ошибка: '-log' требует наличие аргумента
9 Имя файла журнала
10 Записывается в 
11 ошибка: '-rc' требует наличие аргумента
12 ошибка: '-screen' требует наличие аргумента
13 Fluxbox %s: (c) %s Fluxbox Team\n\
Веб-сайт: http://www.fluxbox.org/\n\n\
-display <string>\t\tиспользовать заданный дисплей.\n\
-screen <all|int,int,int>\tзапускать только на определенном экране.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tиспользовать альтернативный файл ресурсов.\n\
-version\t\t\tвывести номер версии и выйти.\n\
-info\t\t\t\tвывести некоторую полезную информацию и выйти.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tжурналировать вывод в указанный файл.\n\
-help\t\t\t\tвывести эту подсказку и выйти.\n\n
14 предупреждение: не удалось установить переменную окружения 'DISPLAY'

$set 20 #Layer

1 Поверх дока
2 Ниже
3 Рабочий стол
4 Док
5 Нормальный
6 Поверх

