$ codeset=ISO-8859-15

$set 1 #Align

1 Abajo al centro
2 Abajo a la izquierda
3 Abajo a la derecha
4 Horizontal
5 A la izquierda
6 A la izquierda abajo
7 A la izquierda al centro
8 A la izquierda arriba
9 Relativo
10 A la derecha
11 A la derecha abajo
12 A la derecha al centro
13 A la derecha arriba
14 Arriba al centro
15 Arriba a la izquierda
16 Arriba a la derecha
17 Vertical

$set 2 #BaseDisplay

1 Abortando... volcando nucleo\n
2 Apagando\n
3 %s:       señal %d capturada\n

$set 3 #Common

1 Alfa
2 Auto ocultar
3 Compilado
4 Opciones compiladas
5 Compilador
6 Versión del compilador
7        init
8        keys
9         nls
10        menu
20  windowmenu
11       style
12 Opciones por defecto
13 desactivado
14 Error
15 Versión de Fluxbox
16 Maximizar por encima
17 Revisión de GIT
18 Visible
19 No hay una opción de fondo especificada en este estilo.\nPor vafor consulte el manual o lea las FAQ.

$set 4 #Configmenu

1 Anti alias
2 Elevar automáticamente las ventanas
3 Seleccionar para elevar
4 Seleccionar para enfocar
5 Decorar las ventanas transitivas
6 Cambiar de escritorio con la rueda del mouse
7 Enfocar última ventana en el cambio de escritorio
8 Modelo de enfoque
9 Enfocar ventanas nuevas
10 Maximización total
11 Simulación de colores en imágenes
12 Movimiento opaco de ventanas
14 El mouse enfoca
15 Workspace Warping
16 Forzar pseudo transparencias
17 Alfa del menu
18 Transparencia
19 Alfa de ventana enfocada
20 Alfa de ventana desenfocada
21 Opciones de solapas
22 Solapas en barra de título
23 Ancho de solapas externas
24 Click sobre la solapa enfoca
25 Mouse sobre la solapa enfoca

$set 5 #Ewmh

1 Fatal: Sin memoria, no se puede asignar la lista de clientes EWMH

$set 6 #FbTkError

1 ¡Atención! Altura > 3200, estableciendo Altura = 3200
2 ¡Atención! Ancho > 3200, estableciendo Ancho = 3200
3 Atención: no se puede cargar la fuente
4 Error de asignación.
5 Error creando el pixmap
6 No se puede crear XImage
7 Error de lectura
8 Atención: No se puede encontrar un formato válido para el alfa
9 Atención: Error al crear la imagen alfa
10 Atención: Error al crear el pixmap alfa
11 No se puede encontrar un formato para la pantalla(%d)
12 Sin memoria
13 Sin memoria mientras se asignaba buffer azul.
14 Sin memoria mientras se asignaba buffer verde.
15 Sin memoria mientras se asignaba buffer rojo.
16 Error de lectura en ítem de tema
17 visual no soportada
18 Estableciendo el valor por defecto

$set 7 #Fluxbox

1 nombre de archivo rc inválido
2 Error al cargar archivo group
3 Error al cargar la base de datos
4 Reintentando con
5 Atención! No se puede encontrar pantalla para mapear la ventana
6 No se puede crear el directorio %s
7 No se pudieron encontrar pantallas para manejar.\nAsegúrese que no tiene otro gestor de ventanas ejecutándose
8 Error interpretando expresión regular
9 Error Fatal! Sólo puede haber una instancia de clase de FLuxbox.
10 No se puede conectar con el servidor X.\nAsegúrese de iniciar las X antes de iniciar Fluxbox
11 Atención: el servidor X no soporta sus locales
12 Atención: no se pueden establecer modificadores de locales

$set 8 #Gnome

1 Error Fatal: Sin memoria, no se puede asignar la lista de clientes de GNOME

$set 9 #Keys

1 Keys: Error en la línea
2 Keys: Error al unir el árbol de teclas!
3 Keys: Tecla/modificador inválido en la línea

$set 10 #Menu

1 Configuración
2 Menú Fluxbox por defecto
3 Salir
4 Iconos
5 Capa...
6 A la cabeza...
7 Ubicación
8 Recargar configuración
9 Reiniciar
10 Atención: etiquetas desbalanceadas de [codificación]

$set 11 #Remember

1 Decoraciones
2 Dimensiones
3 Saltar al escritorio
4 Capa
5 Recordar...
6 Posición
7 Guardar al cerrar
8 Enrollado
9 Pegajoso
10 Tecla desconocida de aplicaciones
11 Escritorio
12 Cabeza
13 Transparencia

$set 12 #Screen

1 BScreen::BScreen: Ocurrió un error al consultar al servidor X.\n     ya existe un window manager connectado al display
2 W: %4d x H: %4d
3 BScreen::BScreen: manejando pantalla %d utilizando visual 0x%lx, profundidad %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 Clientes
2 Ciclar hacia abajo
3 Ciclar hacia arriba
4 Dirección de slit
5 Capa de slit
6 Slit a la cabeza
7 Ubicación de slit
8 Slit
9 Guardar lista de slit

$set 14 #Toolbar

1 Renombrar escritorio actual
2 Modo de la barra de iconos
3 Todas las ventanas
4 Iconos
5 Ninguno
6 Escritorio
7 Iconos del escritorio
8 Capa de la barra de herramientas
9 Toolbar a la cabeza
10 Ubicación de la barra de herramientas
11 Barra de herramientas
12 Porcentaje de ancho de la barra de herramientas
13 Reloj: 24h
14 Reloj: 12h
15 Editar el formato de reloj
16 Mostrar imágenes
17 Sin iconos
18 No mostrar los iconos del escritorio

$set 15 #Window

1 Sin nombre

$set 16 #Windowmenu

1 Cerrar
2 Iconificar
3 Capa
4 Bajar
5 Maximizar
6 Elevar
7 Enviar a ...
8 Enrollar
9 Fijar
10 Matar
11 Utilizar valores por defecto

$set 17 #Workspace

1 Escritorio %d
2 Escritorios
3 Escritorio nuevo
4 Quitar el último escritorio

$set 18 #fbsetroot

1 Error: es necesario especifiar uno de: -solid, -mod, -gradient\n
2 No se pudieron crear los atomos de pixmaps, me rindo!
3 -display <string>        conectar a un display\n\
-mod <x> <y>             diseño de modula\n\
-foreground, -fg <color> color de primer plano\n\
-background, -bg <color> color de fondo\n\n\
-gradient <texture>      gradiente de textura\n\
-from <color>            color inicial del gradiente\n\
-to <color>              color final del gradiente\n\
-solid <color>           color sólido\n\n\
-help                    mostrar este texto de ayuda y salir\n

$set 19 #main

1 error: '-display' requiere un argumento
2 Asignación Inválida
3 Molde Inválido
4 Fuera de rango
5 Error de rutina
6 Execepción estándar
7 Error desconocido
8 error: '-log' requiere un argumento
9 Archivo de registro
10 Guardando registro en
11 error: '-rc' requiere un argumento
12 error: '-screen' requiere un argumento
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
Página web: http://www.fluxbox.org/\n\n\
-display <string>\t\tutilizar conexion a un diplay.\n\
-screen <all|int,int,int>\tejecutar solamente en pantallas especificadas.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tusar archivo de recursos alternativo.\n\
-version\t\t\tmostrar versión y salir.\n\
-info\t\t\t\tmostrar información útil.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tregistrar salidas en archivo.\n\
-help\t\t\t\tmostrar esta ayuda y salir.\n\n
14 atención: no se pudo establecer la variable de entorno 'DISPLAY'

$set 20 #layers

1 Sobre dock
2 Abajo
3 Escritorio
4 Dock
5 Normal
6 Arriba


