$ codeset=ISO-8859-15

$set 1 #Align

1 Onder midden
2 Onder links
3 Onder rechts
4 Horizontaal
6 Links onder
7 Links midden
8 Links boven
11 Rechts onder
12 Rechts midden
13 Rechts boven
14 Boven midden
15 Boven links
16 Boven rechts
17 Vertikaal

$set 2 #BaseDisplay


$set 3 #Common

2 Automatisch verbergen

$set 4 #Configmenu

1 Lettertypes bijwerken
2 Automatisch verhogen
4 Klik om Focus te krijgen
7 Focus Vensters na werkveldwissel
8 Focussoort
9 Nieuwe vensters focussen
10 Volledig maximaliseren
11 Plaatjes afvlakking
12 Venster ondoorzichtig verplaatsen
13 Bijna altijd muis focus
14 Muis focus
15 Werkveld springen

$set 5 #Ewmh


$set 6 #FbTkError


$set 7 #Fluxbox


$set 8 #Gnome


$set 9 #Keys


$set 10 #Menu

3 Afsluiten
4 Iconen
7 Plaats
9 Herstart

$set 11 #Remember


$set 12 #Screen

2 W: %4d x H: %4d
4 W: %04d x H: %04d

$set 13 #Slit

4 Spleet-Richting
7 Spleet-Plaatsing
8 Spleet

$set 14 #Toolbar

1 Werkveldnaam veranderen
10 Werkbalkplaatsing
11 Werkbalklijst

$set 15 #Window

1 Naamloos

$set 16 #Windowmenu

1 Sluiten
2 Minimalizeren
4 Onderop leggen
5 Maximalizeren
6 Bovenop leggen
7 Stuur naar
8 Oprollen
9 Plakkerig

$set 17 #Workspace

1 Werkveld %d
2 Werkvelden
3 Nieuw werkveld
4 Verwijder laatste

$set 18 #fbsetroot

1 error: must specify one of: -solid, -mod, -gradient\n
3 -display <string>        display connection\n\
-mod <x> <y>             modula pattern\n\
-foreground, -fg <color> modula foreground color\n\
-background, -bg <color> modula background color\n\n\
-gradient <texture>      gradient texture\n\
-from <color>            gradient start color\n\
-to <color>              gradient end color\n\n\
-solid <color>           solid color\n\n\
-help                    print this help text and exit\n

$set 19 #main

1 error: '-display' heeft een argument nodig
11 error: '-rc' heeft een argument nodig
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tkies ander scherm.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tgebruik alternatieve configuratie.\n\
-version\t\t\ttoon versienummer.\n\
-info\t\t\t\ttoon bruikbare informatie.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tlog uitvoer naar bestand.\n\
-help\t\t\t\ttoon deze hulptekst.\n\n


