$ codeset=ISO-8859-2

$set 1 #Align

1 Dolny Środkowy
2 Dolny Lewy
3 Dolny Prawy
4 Poziomo
5 Z Lewej
6 Dolny Lewy
7 Środkowy Lewy
8 Górny Lewy
9 Względny
10 Z Prawej
11 Dolny Prawy
12 Środkowy Prawy
13 Prawy Górny
14 Górny Środkowy
15 Górny Lewy
16 Górny Prawy
17 Pionowo

$set 2 #BaseDisplay

1 Porzucam...Zrzucam pamięć\n
2 Zamykam\n
3 %s:    sygnał %d otrzymał\n

$set 3 #Common
1 Przeźroczystość
2 Auto ukrywanie
3 Skompilowano
4 Skompilowano z opcjami
5 Kompilator
6 Wersja kompilatora
7 init
8 keys
10 menu
11 Styl
12 Wartości domyślne
13 wyłączone
14 Błąd
15 Fluxbox wersja
16 Maksymalizuj nad
18 Widoczny



$set 4 #Configmenu

1 AntiAlias
2 Automatyczne podnoszenie
3 Kliknij aby dać na wierzch
4 Kliknij by uaktywnić
5 Dekoracje na oknach-dzieciach
6 Zmiana pulpitów kółkiem myszy
7 Zmień aktywne okno przy zmianie pulpitów
8 Sposób aktywacji okien
9 Ustawiaj nowe okna jako aktywne
10 Pełna maksymalizacja
11 Rastrowanie Obrazu
12 Wyświetlaj zawartość przesuwanego okna
13 Pół-automatyczna aktywacja
14 Pokaż kursorem by uaktywnić
15 Pulpity obok siebie

16 Wymuś pseudo przeźroczystość
17 Przeźroczystość menu
18 Przeźroczystość
19 Przeźroczystość aktywnego okna
20 Przeźroczystość nieaktywnego okna

$set 5 #Ewmh

1 Błąd krytyczny: Brak pamięci, nie mogę zarezerwować lity klientów EWMH

$set 6 #FbTkError

1 Ostrzeżenie! Wysokość > 3200 Ustawiam Wysokość = 3200
2 Ostrzeżenie! Szerokość > 3200 Ustawiam Szerokość = 3200
3 Ostrzeżenie: nie mogę załadować zapasowej czcionki
4 Błąd allokacji.
5 Błąd podczas tworzenia pixmapy
6 Nie mogę stworzyć XImage
7 Błąd odczytu
8 Ostrzeżenie: Nie znaleziono poprawnych ustawień przeźroczystości
9 Ostrzeżenie: Nie powiodło się tworzenie przeźroczystego obrazu
10 Ostrzeżenie: Nie powiodło się tworzenie przeźroczystej pixmapy 
11 Nie znalałem formatu dla ekranu(%d)
12 Brak pamięci
13 Brak pamięci podczas alokacji niebieskiego bufora.
14 Brak pamięci podczas alokacji zielonego bufora.
15 Brak pamięci podczas alokacji czerwonego bufora.
16 Nie rozpoznaję elementu stylu
17 błąd podczas renderowania obrazka
18 Ustawiam wartość domyślną

$set 7 #Fluxbox

1 niewłasciwa nazwa pliku zasobów!
2 Nie mogę załadować pliku grup
3 Nie mogę załadować bazy danych
4 Próbuje ponownie z
5 Ostrzeżenie! Nie mogę znaleźć ekranu na który mogę zmapować okno!
6 Nie mogę utworzyć katalogu %s
7 Nie moge znaleźć ekranów do zarządzania\nUpewnij się że nie masz włączonego inne menedźera okien.
8 Błąd podczas parsowania wyrażenia reguralnego
9 Błąd krytyczny! Może istnieć tylko jedna instancja klasy fluxbox.
10 Nie mogę połączyć się z serwerem X.\nUpewnij się że uruchomiłeś X'y przed uruchomieniem Fluxbox.
11 Ostrzeżenie: serwer X nie wspiera ustawień lokalnych
12 Ostrzeżenie: nie można ustawić modyfikatorów lokalnych

$set 8 #Gnome

1 Błąd krytyczny: Brak pamięci, nie mogę zaalokować dla listy klientów GNOME

$set 9 #Keys

1 Keys: Błąd w lini
2 Keys: Błąd podczas łączenia drzew klawiszy!
3 Keys: Niewłaściwy klawisz/modyfikator w linii

$set 10 #Menu

1 Konfiguracja
2 Domyślne menu fluxbox
3 Wyjście
4 Ikonki
5 Warstwa...
6 Na monitorze...
7 Umiejscowienie
8 Przeładuj ustawienia
9 Restart


$set 11 #Remember

1 Dekoracje
2 Rozmiar
3 Przejdź do pulpitu
4 Warstwa
5 Zapamiętaj
6 Pozycja
7 Zapisz przy wyjściu
8 Zwinięty
9 Przyklejony
10 Nieznany klucz pliku apps
11 Pulpit

12 Monitor

$set 12 #Screen

1 BScreen::BScreen: wystąpił błąd podczas wysyłania zapytania do serwera X'ów.\nNa ekranie włączony jest już inny menedżer okien
2 S: %4d x W: %4d
4 S: %04d x W: %04d

3 BScreen::BScreen: używanie ekrany %d używam visuala 0x%lx, głębokość %d\n


$set 13 #Slit

1 Klienci
2 Przesuń wszystkich w górę
3 Przesuń wszystkich w dół
4 Kierunek Tacki
5 Warstwa
6 Na monitorze
7 Położenie Tacki
8 Tacka

9 Zapamiętaj kolejność

$set 14 #Toolbar

1 Zmień nazwę aktywnego pulpitu
2 Tryb paska ikon
3 Wszystkie okna
4 Ikony
5 Żadne
6 Pulpit
7 Ikony Pulpitu
8 Warstwa Paska Narzędzi
10 Położenie
11 Pasek Narzędzi
12 Szerokość w procentach

13 Zegar w trybie 24h
14 Zegar w trybie 12h
15 Format wyświetlania zegara
16 Pokaż ikony

$set 15 #Window

1 Bez nazwy

$set 16 #Windowmenu

1 Zamknij
2 Minimalizuj
3 Warstwa
4 Pod spód
5 Maksymalizuj
6 Na wierzch
7 Wyślij na ...
8 Zwiń
9 Przyklej

10 Zakończ

$set 17 #Workspace

1 Pulpit %d
2 Pulpity
3 Nowy Pulpit
4 Usuń Ostatni

$set 18 #fbsetroot

1 błąd: musisz określić jedno z: -solid, -mod, -gradient\n
2 Nie mogę stowrzyć elementów pixmap - daję sobie spokój!
3 -display <string>        korzystam z ekranu <nazwa ekranu>\n\
-mod <x> <y>             wzór modułu\n\
-foreground, -fg <kolor> kolor pierwszego planu modułu\n\
-background, -bg <kolor> kolor tła modułu\n\n\
-gradient <tekstura>     cieniowana tekstura\n\
-from <kolor>            pierwszy kolor cieniowania\n\
-to <kolor>              drugi kolor cieniowania\n\n\
-solid <kolor>           stały kolor\n\n\
-help                    drukuję tę pomoc i wychodzę\n

$set 19 #main

1 błąd: '-display' wymaga argumentu
2 Zła allokacja
4 Poza zakresem
5 Błąd wykonania
6 Standardowy wyjątek
7 Nieznany błąd
8 Błąd: '-log' wymaga wartości
9 Plik dziennika
10 Zapisuję dziennik
11 błąd: '-rc' wymaga argumentu
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tużyj połączenia ekranu.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tużyj innego pliku zasobów.\n\
-version\t\t\twyświetlam wersję i wychodzę.\n\
-info\t\t\t\twyświetlam przydatne informacje.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tlogi przesyłam do pliku.\n\
-help\t\t\t\twyświetlam tę pomoc i wychodzę.\n\
14 Nie mogłem ustawić zmiennej środowiskowej 'DISPLAY'\n\n
 

$set 20 #layers

1 Powyżej docku
2 Na dno
3 Pulpit
4 Dock
5 Normalna
6 Na wierzch


