$ codeset=ISO-8859-1

$set 1 #Align

1 Em Baixo no Centro
2 Em Baixo à Esquerda
3 Em Baixo à Direita
4 Horizontal
5 Esquerda
6 À Esquerda em Baixo
7 À Esquerda no Centro
8 À Esquerda em Cima
9 Relativo
10 Direita
11 À Direita em Baixo
12 À Direita no Centro
13 À Direita em Cima
14 Em Cima no Centro
15 Em Cima à Esquerda
16 Em Cima à Direita
17 Vertical

$set 2 #BaseDisplay

1 Interrompendo... criando core\n
2 Desligando\n
3 %s:      sinal %d recebido\n

$set 3 #Common

1 Transparência
2 Esconder Automaticamente
3 Compilado
4 Opções de Compilação
5 Compilador
6 Versão do Compilador
7        init
8        keys
9         nls
10        menu
20  windowmenu
11       style
12 Por Omissão
13 desactivado
14 Erro
15 Versão do Fluxbox
16 Maximizar Sobre
17 Revisão GIT
18 Visível
19 Não existe opção de Fundo neste tema.\nConsulte o manual ou as FAQ.

$set 4 #Configmenu

1 AntiAlias
2 Sobrepor Janela Quando Activa
3 Clicar Sobrepõe Janela
4 Foco na Janela Quando Clicada
5 Decoração em Janelas Temporárias
6 Roda do Rato muda Área de Trabalho
7 Foco na Última Janela Da Área de Trabalho
8 Modelo de Foco
9 Foco em Novas Janelas
10 Maximização Total
11 Simulação de Cores em Imagens
12 Movimento Opaco de Janelas
14 Foco na Janela Quando Apontada
15 Áreas de Trabalho Contínuas
16 Forçar Pseudo-Transparência
17 Transparência do Menu
18 Transparência
19 Transparência de Janelas com Foco
20 Transparência de Janelas sem Foco
21 Opções de Abas
22 Abas na Barra de Título
23 Dimensão de Abas Externas
24 Foco na Aba Quando Apontada
25 Foco na Aba Quando Clicada

$set 5 #Ewmh

1 Fatal: Falta de memória, não é possível alocar lista de clientes para EWMH

$set 6 #FbTkError

1 Aviso! Altura > 3200, definindo Altura = 3200
2 Aviso! Largura > 3200, definindo Largura = 3200
3 Aviso: não foi possível abrir fonte de recurso
4 Erro de alocação.
5 Erro na criação de pixmap
6 Não é possível criar XImage
7 Falha de leitura
8 Aviso: Não foi possível encontrar um formato válido para transparência.
9 Aviso: Não foi possível criar uma imagem transparente.
10 Aviso: Não foi possível criar um pixmap transparente.
11 Não foi possível encontrar o formato do ecrã(%d)
12 Sem memória
13 Sem memória ao alocar buffer de azul.
14 Sem memória ao alocar buffer de verde.
15 Sem memória ao alocar buffer de vermelho.
16 Falha de leitura de item de tema
17 visual não suportado
18 Definindo valor por defeito

$set 7 #Fluxbox

1 Ficheiro de configuração é inválido!
2 Não foi possível abrir ficheiro de grupos
3 Não foi possível abrir a base de dados
4 Tentar de novo com
5 Aviso! Não foi encontrado um ecrã onde mapear a janela!
6 Não é possível criar directoria %s
7 Não foram encontrados ecrãs para gerir.\nAssegure-se de que não tem outro gestor de janelas activo. 
8 Erro no tratamento de expressão regular
9 Fatal! Só é possível haver uma instância da classe fluxbox.
10 Não é possível ligar ao servidor X.\nAssegure-se de que inicia o X antes do Fluxbox.
11 Aviso: Servidor X não suporta traduções
12 Aviso: Não é possível definir modificadores localizados

$set 8 #Gnome

1 Fatal: Sem memória, não é possível alocar lista de clientes para GNOME

$set 9 #Keys

1 Teclas: Erro na linha
2 Teclas: Falha ao fundir àrvore de atalhos de teclas!
3 Teclas: Tecla/Modificador inválido na linha

$set 10 #Menu

1 Configuração
2 Menu por Defeito do Fluxbox
3 Sair
4 Janelas Minimizadas
5 Camada...
6 No Monitor...
7 Posicionamento
8 Recarregar Configurações
9 Reiniciar
10 Aviso: definição [encondig] não-balanceada

$set 11 #Remember

1 Decorações
2 Dimensões
3 Ir para Área de Trabalho
4 Camada
5 Lembrar...
6 Posição
7 Gravar ao sair
8 Enrolado
9 Sempre Visível
10 Item de Aplicação Desconhecido
11 Área de Trabalho
12 Monitor
13 Transparência

$set 12 #Screen

1 BScreen::BScreen: ocorreu um erro ao interrogar o servidor X.\n outro gestor de janelas está a correr
2 L: %4d x A: %4d
3 BScreen::BScreen: gerindo ecrã %d usando visual 0x%lx, profundidade %d\n
4 L: %04d x A: %04d

$set 13 #Slit

1 Clientes
2 Rodar para Baixo
3 Rodar para Cima
4 Direcção
5 Camada
6 Monitor
7 Posicionamento
8 Slit
9 Gravar Slitlist

$set 14 #Toolbar

1 Editar o nome da Área de Trabalho
2 Modo da Barra de Janelas
3 Todas as Janelas
4 Apenas Janelas Minimizadas
5 Nenhuma
6 Área de Trabalho
7 Apenas Janelas Minimizadas da Área de Trabalho
8 Camada da Barra de Ferramentas
9 Barra de Ferramentas no Monitor
10 Posicionamento
11 Barra de Ferramentas
12 Dimensão em percentagem
13 Relógio: 24h
14 Relógio: 12h
15 Editar Formato do Relógio
16 Mostrar Imagens
17 Todas Excepto Janelas Minimizadas
18 Sem Janelas Minimizadas da Área de Trabalho

$set 15 #Window

1 Sem Nome

$set 16 #Windowmenu

1 Fechar
2 Minimizar
3 Camada
4 Desfocalizar
5 Maximizar
6 Focalizar
7 Enviar Para ...
8 Enrolar
9 Sempre Visível
10 Forçar Terminar
11 Usar Valores Padrão

$set 17 #Workspace

1 Área de Trabalho %d
2 Áreas de Trabalho
3 Nova Área de Trabalho
4 Remover Última Área de Trabalho

$set 18 #fbsetroot

1 Erro: é necessário especificar um dos seguintes argumentos: -solid, -mod, -gradient\n
2 Não foi possível criar átomos de pixmaps, desistindo!
3 -display <string>        ligação com display\n\
-mod <x> <y>             altera padrão\n\
-foreground, -fg <color> altera cor de primeiro plano\n\
-background, -bg <color> altera cor de fundo\n\n\
-gradient <texture>      textura do gradiente\n\
-from <color>            cor de início do gradiente\n\
-to <color>              cor do fim do gradiente\n\n\
-solid <color>           cor sólida\n\n\
-help                    mostra esta ajuda e sai\n

$set 19 #main

1 Erro: '-display' requer um argumento
2 Erro de Alocação
3 Erro de Conversão
4 Fora dos Limites
5 Erro de Execução
6 Excepção Standard
7 Erro desconhecido
8 Erro: '-log' necessita um argumento
9 Ficheiro de Registo
10 Guardando Registo em
11 Erro: '-rc' necessita de um argumento
12 Erro: '-screen' necessita de um argumento
13 Fluxbox %s: (c) %s Fluxbox Team\n\
Website: http://www.fluxbox.org/\n\n\
-display <string>\t\tusar ligação com o display.\n\
-screen <all|int,int,int>\tcorrer nos ecrãs especificados apenas.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tusar arquivo alternativo de recursos.\n\
-version\t\t\tmostrar versão e sair.\n\
-info\t\t\t\tmostrar alguma informação útil.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tguarda registos para um ficheiro.\n\
-help\t\t\t\tmostrar esta ajuda e sair.\n\n
14 aviso: não foi possível definir a variável de ambiente 'DISPLAY'

$set 20 #layers

1 Acima de Doca
2 Em Baixo
3 Desktop
4 Doca
5 Normal
6 Topo


