$ codeset=UTF-8

$set 1 #Align

1 All keskel
2 All vasakul
3 All paremal
4 Horisontaalne
6 Left Bottom
7 Left Center
8 Left Top
13 Right Top
14 Üleval keskel
15 Üleval vasakul
16 Üleval paremal
17 Vertikaalne

$set 2 #BaseDisplay


$set 3 #Common

2 Auto hide

$set 4 #Configmenu

2 Akende esiletoomine
4 Klikka fookuse saamiseks
7 Fokusseeri viimane aken
8 Fookusmood
9 Fookus uutele akendele
10 Üle-ekraani suurendus
11 Piltide teravustamine
12 Nähtav akende liigutamine
13 Semi Sloppy Focus
14 Kaasaskäiv fookus
15 Workspace Warping

$set 5 #Ewmh


$set 6 #FbTkError


$set 7 #Fluxbox


$set 8 #Gnome


$set 9 #Keys


$set 10 #Menu

3 Välju
4 Ikoonid
7 Right Bottom
9 Restart

$set 11 #Remember


$set 12 #Screen

2 W: %4d x H: %4d
4 W: %04d x H: %04d

$set 13 #Slit

4 Prao suund
7 Prao asukoht
8 Pragu

$set 14 #Toolbar

1 Muuda töölaua nime
10 Tegumiriba asukoht
11 Tegumiriba

$set 15 #Window

1 Nimeta

$set 16 #Windowmenu

1 Sulge
2 Ikoniseeri
4 Tahaplaanile
5 Suurenda
6 Esiplaanile
7 Saada...
8 Varjuta
9 Kleebi

$set 17 #Workspace

1 Töölaud %d
2 Töölauad
3 Uus töölaud
4 Eemalda viimane

$set 18 #fbsetroot

1 viga: pead määratlema ühe järgnevaist: -solid, -mod, -gradient\n
3 -display <string>        displei ühendus\n\
-mod <x> <y>             moodula tapeet\n\
-foreground, -fg <color> esiplaani värvi moodula\n\
-background, -bg <color> tausta värvi moodula\n\n\
-gradient <texture>      gradientne tekstuur\n\
-from <color>            gradiendi algusvärv\n\
-to <color>              gradiendi lõpuvärv\n\n\
-solid <color>           ühtne värv\n\n\
-help                    näida seda abiteksti ja välju\n

$set 19 #main

1 viga: '-display' vajab argumenti
11 viga: '-rc' vajab argumenti
13 Fluxbox %s: (c) %s Fluxbox Team\n\
\n\
-display <string>\t\tkasuta displei ühendust.\n\
-rc <string>\t\t\tkasuta alternatiivseid resursse.\n\
-version\t\t\tnäita versiooninumber ja välju.\n\
-info\t\t\t\tdisplay some useful information.\n\
-log <filename>\t\t\tlog output to file.\n\
-help\t\t\t\tnäita seda abiteksti ja välju.\n\n

