$ codeset=ISO-8859-1

$set 1 #Align

1 Bunden i midten
2 Bunden til venstre
3 Bunden til højre
4 Horisontal
5 Til venstre
6 Til venstre i bunden
7 Til venstre i midten
8 Til venstre i toppen
9 Relativ
10 Til højre
11 Til højre i bunden
12 Til højre i midten
13 Til højre i toppen
14 Toppen i midten
15 Øverste venstre hjørne
16 Øverste højre hjørne
17 Vertikal

$set 2 #BaseDisplay

1 Afslutter brat... laver et core dump\n
2 Lukker ned\n
3 %s:      signal %d fanget\n

$set 3 #Common

1 Alfa
2 Skjul automatisk
10 menu
11 stil
12 Standarder
13 Slået fra
14 Fejl
15 Fluxbox version
16 Maksimer over
17 GIT Revision
18 Synlig

$set 4 #Configmenu

1 Antialias
2 Hæv automatisk
3 Klik hæver
4 Klik for fokus
5 Dekorér transiente vinduer
6 Skift skrivebord med scroll-hjulet
7 Fokuser på vindue ved skrivebordsskift
8 Fokusmodel
9 Fokuser på nye vinduer
10 Fuld maksimering
11 Billede-dithering
12 Ugennemsigtig flytning af vinduer
13 Semisløset Focus
14 Dovent fokus
15 Workspace Warping
16 Forcér pseudogennemsigtighed
18 Gennemsigtighed

$set 5 #Ewmh

1 Fatal fejl: Løbet tør for hukommelse, ikke nok til at allokere EWMH's klientliste.

$set 6 #FbTkError

1 Advarsel! Højde > 3200, sætter højde = 3200
2 Advarsel! Bredde > 3200, sætter bredde = 3200
3 Advarsel: Kan ikke indlæse reserveskrifttype
4 Allokeringsfejl.
5 Fejl i oprettelse af pixmap
6 Kunne ikke oprette XImage (`Xbillede')
7 Kunne ikke læse.
8 Advarsel: Fandt ikke noget gyldigt format for alfa.
9 Advarsel: Kunne ikke oprette alfa-billede.
10 Advarsel: Kunne ikke oprette alfa-pixmap.
11 Kunne ikke finde format for skærm(%d)
12 Løbet tør for hukommelse
13 Løbet tør for hukommelse under allokering af blå buffer.
14 Løbet tør for hukommelse under allokering af grøn buffer.
15 Løbet tør for hukommelse under allokering af rød buffer.
16 Kunne ikke læse tema-object
17 Uunderstøttet visual
18 Sætter standardværdi

$set 7 #Fluxbox

1 rc-filnavn ugyldigt!
2 Kunne ikke indlæse gruppefilen
3 Kunne ikke indlæse databasen
4 Prøver igen med
6 Kan ikke oprette mappen %s
7 Kunne ikke finde skærme til håndtering.\nSørg for at ikke at ha' en anden window manager kørende.
8 Fejl: kan ikke parse regulært udtryk
9 Fatal fejl: der kan kun være en instans af klassen fluxbox.
10 Kunne ikke forbinde til X-serveren.\nSørg for at starte X før du starter Fluxbox.
11 Advarsel: X-serveren understøtter ikke lokale indstillinger (locale)
12 Advarsel: Kan ikke sætte `locale'-modifikatorer

$set 8 #Gnome

1 Fatal fejl: Løbet tør for hukommelse, ikke nok til at allokere GNOME's klientliste.

$set 9 #Keys

1 Taster (Keys): Fejl på linie
2 Taster (Keys): kunne ikke sammenflette tastetræ!
3 Taster (Keys): Ugyldig tast på linie

$set 10 #Menu

1 Indstillinger
2 Fluxbox standardmenu
3 Afslut
4 Ikoner
5 Lag...
7 Placering
8 Genindlæs indstillinger
9 Genstart

$set 11 #Remember

1 Dekorationer
2 Størrelse
3 Gå til skriverbord
4 Lag
5 Husk...
6 Placering
7 Gem ved lukning
8 Kun titelbjælke
9 På alle skriveborde
10 Ukendt apps nøgle
11 Skrivebord

$set 12 #Screen

1 BScreen::BScreen: der opstod en fejl under forespørgsel til X server.\n	en anden window manager kører allerede på display
2 B: %4d x H: %4d
3 BScreen::BScreen: håndterer skærm %d med visual 0x%lx, dybde %d\n
4 B: %04d x H: %04d

$set 13 #Slit

1 Klienter
4 Slidsretning
5 Slidslag
7 Slidsplacering
8 Slids
9 Gem slidsliste

$set 14 #Toolbar

1 Rediger skrivebordets navn
2 Vinduer vist i væjktøjsbjælken
3 Alle
4 Minimerede
5 Ingen
6 På skrivebord
7 Minimerede på skrivebord
8 Værktøjsbjælkelag
10 Værktøjsbjælkens placering
11 Værktøjsbjælke
13 Ur: 24 timer
14 Ur: 12 timer
15 Rediger tidsformat
16 Vis billeder
17 Ikke-minimerede
18 Ikke-minimerede på skrivebord

$set 15 #Window

1 Intet navn

$set 16 #Windowmenu

1 Luk
2 Minimeret
3 Lag
4 Sænk
5 Maksimer
6 Hæv
7 Send til ...
8 Kun titelbjælke
9 På alle skriveborde
10 Luk abrupt

$set 17 #Workspace

1 Skrivebord %d
2 Skriveborde
3 Nyt skrivebord
4 Fjern sidste

$set 18 #fbsetroot

1 fejl: specificér en af følgende: -solid, -mod, -gradient\n
3 -display <string>        skærmtilslutning\n\
-mod <x> <y>             modula mønster\n\
-foreground, -fg <color> modula foregrundsfarve\n\
-background, -bg <color> modula baggrundsfarve\n\n\
-gradient <texture>      gradienttekstur\n\
-from <color>            gradientens startfarve\n\
-to <color>              gradientens slutfarve\n\n\
-solid <color>           fast farve\n\n\
-help                    vis denne hjælp og afslut\n

$set 19 #main

1 fejl: '-display' kræver et argument
2 Dårlig allokering
7 Ukendt fejl
9 Logfil
10 Logger til
11 fejl: '-rc' kræver et argument
12 error: '-screen' requires an argument
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tbrug skærmtilslutning.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tbrug alternativ resourcefil.\n\
-version\t\t\tvis versionsnummer og afslut.\n\
-info\t\t\t\tvis noget nyttig information.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tlog output til fil.\n\
-help\t\t\t\tvis denne hjælp og afslut.\n\n
14 warning: couldn't set environment variable 'DISPLAY'

$set 20 #layers

1 Over dok
2 Nederst
3 Skrivebord
4 Dok
5 Normal
6 Øverst
