$ codeset=UTF-8

$set 1 #Align

1 Na dnu in osredinjeno
2 Na dnu in levo poravnano
3 Na dnu in desno poravnano
4 Vodoravno
6 Levo spodaj
7 Levo v sredini
8 Levo zgoraj
11 Desno spodaj
12 Desno v sredini
13 Desno zgoraj
14 Na vrhu in osredinjeno
15 Na vrhu in levo poravnano
16 Na vrhu in desno poravnano
17 Navpično

$set 2 #BaseDisplay


$set 3 #Common

2 Samodejno skrivanje

$set 4 #Configmenu

1 Glajenje pisav
2 Samodejno odkrivanje
4 Za fokus kliknite
7 Spremenjeni fokus okna na namizju
8 Način fokusiranja
9 Novo okno dobi fokus
10 Največje okno
11 Prelivanje slik
12 Premikanje prosojnih oken
13 Napol površno fokusiranje
14 Površni fokus
15 Premikanje oken na druge delovne površine

$set 5 #Ewmh


$set 6 #FbTkError


$set 7 #Fluxbox


$set 8 #Gnome


$set 9 #Keys


$set 10 #Menu

3 Izhod
4 Ikone
7 Lega
9 Ponovni zagon

$set 11 #Remember


$set 12 #Screen

2 W: %4d x H: %4d
4 W: %04d x H: %04d

$set 13 #Slit

4 Smer reže
7 Lega reže
8 Reža

$set 14 #Toolbar

1 Uredi ime trenutnega namizja
10 Lega orodne letve
11 Orodna letev

$set 15 #Window

1 Neimenovano

$set 16 #Windowmenu

1 Zapusti
2 Pomanjšaj
4 Zakrij
5 Povečaj
6 Prikaži
7 Na delovno površino...
8 Zastri
9 Prilepi

$set 17 #Workspace

1 Delovna površina %d
2 Delovne površine
3 Nova delovna površina
4 Odstrani zadnjo

$set 18 #fbsetroot

1 napaka: določiti je treba eno: -solid, -mod, -gradient\n
3 -display <string>        prikaži povezavo\n\
-mod <x> <y>             vzorec\n\
-foreground, -fg <color> barva ospredja\n\
-background, -bg <color> barva ozadja\n\n\
-gradient <texture>      prelivna tekstura\n\
-from <color>            začetna barva preliva\n\
-to <color>              končna barva preliva\n\n\
-solid <color>           enotna barva\n\n\
-help                    prikaži ta navodila in končaj\n

$set 19 #main

1 napaka: '-display' zahteva argument
11 napaka: '-rc' zahteva argument
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tuporabi prikazovalnik.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tuporabi nadomestno datoteko z viri.\n\
-version\t\t\tprikaži oznako različice in končaj.\n\
-info\t\t\t\tprikaži nekaj uporabnih informacij.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tzapiši dnevnik v datoteko.\n\
-help\t\t\t\tprikaži ta navodila in končaj.\n\n


