$ codeset=UTF-8

$set 1 #Align

1 Унізе па сярэдзіне
2 Унізе леваруч
3 Унізе праваруч
4 Па гарызанталі
5 Леваруч
6 Леваруч ўнізе
7 Леваруч па сярэдзіне
8 Леваруч ўверсе
9 Па усёй шырыні
10 Праваруч
11 Праваруч ўнізе
12 Праваруч па сярэдзіне
13 Праваруч ўверсе
14 Уверсе па сярэдзіне
15 Уверсе леваруч
16 Уверсе праваруч
17 Па вэртыкалі

$set 2 #BaseDisplay

1 Аварыйнае завяршэньне... стварэньне адбітку памяці\n
2 Завяршэньне\n
3 %s:      сыгнал %d атрыманы\n

$set 3 #Common

1 Празрыстасьць
2 Аўтаматычна хаваць
3 Скампіляваны
4 Скампіляваны з выбарам
5 Кампілятар
6 Вэрсія кампілятара
7 Файл наладак
8 Файл клявіш
10 Файл мэню
11 Файл з стылем
12 Дапомныя наладкі
13 выключана
14 Памылка
15 Вэрсія Fluxbox
16 Пад вокнамі
18 Бачная

$set 4 #Configmenu

1 Антыаліасінг
2 Падымаць аўтаматычна
3 Падымаць па шчаўчку
4 Засяроджваньне па шчаўчку
5 Упрыгожваць нерэзыдэнтыя вокны
6 Мяняць стальцы колам мышы
7 Засяроджваньне на вакне пры зьмене працоўнага месца
8 Мадэль засяроджваньня 
9 Засяроджваньне на новых вокнах
10 Максымізаваць цалкам
11 Згладжваньне выяваў
12 Рух запоўненых вокнаў
13 Размытае засяроджваньне
14 Засяроджваньне па руху
15 Рух вокнаў паміж працоўнымі месцамі
16 Змушаць да псэўда-празрыстасьці
17 Празрыстае мэню
18 Празрыстасьць
19 Празрыстае вакно пад засяроджваньнем
20 Празрыстае вакно без засяроджваньня 

$set 5 #Ewmh

1 Згубна: Нестае памяці, немагчыма вылучыць пад EWMH кліент

$set 6 #FbTkError

1 Увага! Вышыня > 3200 Вышыня зробленая = 3200
2 Увага! Шырыня > 3200 Шырыня зробленая = 3200
3 Увага: немагчыма загрузіць запасны шрыфт
4 Памылка вылучэньня.
5 Памылка пры стварэньні выяваў
6 Немагчыма стварыць XImage
7 Немагчыма прачытаць
8 Увага: Немагчыма знайсьці правільны фармат для празрыстасьці.
9 Увага: Немагчыма стварыць празрыстую выяву.
10 Увага: Немагчыма стварыць празрыстую выяву.
11 Немагчыма вызначыць фармат экрану(%d)
12 Нестае памяці
13 Нестае памяці для буфэра блакітнага колеру.
14 Нестае памяці для буфэра зялёнага колеру.
15 Нестае памяці для буфэра чырвонага колеру.
16 Немагчыма прачытаць элемэнт тэмы
17 unsupported visual
18 Усталяваць дапомнасьці

$set 7 #Fluxbox

1 Назва rc-файла няправільная!
2 Немагчыма загрузіць файл груп
3 Немагчыма загрузіць базу дадзеных
4 Паспрабуйце яшчэ з
5 Увага! Немагчыма адшукаць экран каб паказаць акно!
6 Немагчыма стварыць тэчку %s
7 Немагчыма знайсьці экран для кіраваньня.\nУпэўніцеся што не працуе іншы кіраўнік вокнаў.
8 Памылка разбору рэгулярнага выразу
9 Згубна! Можа выконвацца толькі адзін fluxbox .
10 Немагчыма далучыцца да X-паслужніку.\nУпэўніцеся што X стартуе перад Fluxbox.
11 Увага: X-паслужнік не падтрымлівае лякаль
12 Увага: Немагчыма усталяваць мадыфікатары лякалі

$set 8 #Gnome

1 Згубна: Нестае памяці, немагчыма вылучыць пад GNOME кліент

$set 9 #Keys

1 Keys: Памылка ў радку
2 Keys: Збой злучэньня 'keytree'e!
3 Keys: Няправільная кнопка або мадыфікатар у радку

$set 10 #Menu

1 Наладкі
2 Стандартнае мэню Fluxbox
3 Выхад
4 Вобразы
5 Слой...
6 On Head...
7 Месца
8 Перачытаць наладкі
9 Перапуск

$set 11 #Remember

1 Упрыгожваньні
2 Памер
3 Скокнуць да працоўнага месца
4 Слой
5 Памятаць...
6 Пазыцыя
7 Захоўваць пры выхадзе
8 Схаваць у загаловак
9 Прылеплены
10 Unknown apps key
11 Працоўнае месца
12 Загаловак

$set 12 #Screen

1 BScreen::BScreen: памылка у часе зьвяртаньня да X-спаслужніку.\n	другі кіраўнік вокнамі ўжо выконваецца на гэтым дысплэі
2 W: %4d x H: %4d
3 BScreen::BScreen: кіраваньне экранам %d карыстаючыся visual 0x%lx, глыбіня %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 Кліенты
2 Па кругу ўніз
3 Па кругу ўверх
4 Напрамак шчыліны
5 Слой шчыліны
7 Месца шчыліны
8 Шчыліна
9 Захаваць сьпіс шчыліны

$set 14 #Toolbar

1 Рэдагаваць назву гэтага працоўнага месца
2 Рэжым адлюстраваньня вокнаў
3 Усе вокны
4 Вобразы
5 Нічога
6 Працоўнае месца
7 Мінімалізаваныя
8 Слой панэлі прыладаў
9 Toolbar on Head
10 Месца панэлі прыладаў
11 Панэль прыладаў
12 Шырыня ў адсотках
13 Фармат часу: 24 гадзіны
14 Фармат часу: 12 гадзін
15 Заманіць фармат адлюстраваньня часу
16 Паказваць выявы

$set 15 #Window

1 Безназоўнае

$set 16 #Windowmenu

1 Закрыць
2 Зьвярнуць ў панэль прыладаў
3 Слой...
4 Ніжэй
5 Максымізаваць
6 Вышэй
7 Даслаць акно на ...
8 Зьвярнуць ў загаловак
9 Прыляпіць
10 Забіць

$set 17 #Workspace

1 Працоўнае месца %d
2 Працоўныя месцы
3 Новае працоўнае месца
4 Выдаліць апошняе

$set 18 #fbsetroot

1 памылка: павінна быць нешта з наступнага: -solid, -mod, -gradient\n
3 -display <string>        Злучэньне з дысплэям\n\
-mod <x> <y>             modula pattern\n\
-foreground, -fg <color> modula foreground color\n\
-background, -bg <color> modula background color\n\n\
-gradient <texture>      градыент\n\
-from <color>            першы колер градыенту\n\
-to <color>              апошні колер градыенту\n\n\
-solid <color>           суцэльны колер\n\n\
-help                    надрукаваць гэты тэкст ды выйсьці\n

$set 19 #main

1 памылка: '-display' патрэбны аргумэнт
5 Памылка выкананьня
6 Стандартнае выключана
7 Невядомая памылка
8 памылка: '-log' патрэбны аргумэнт
9 Файл часопісу
10 Запісваць часопісу
11 памылка: '-rc' патрэбны аргумэнт
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tкарыстацца адзначаным дасыплем.\n\
-no-slit\t\t\tdo not provide a slit\n\
-no-toolbar\t\t\tdo not provide a toolbar\n\
-rc <string>\t\t\tкарыстацца другім файлам рэсурсаў.\n\
-version\t\t\tпаказаць нумар вэрсіі ды выйсьці.\n\
-info\t\t\t\tпаказаць некаторую карысную інфармацыю ды выйсьці.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tЗапісваць лог у адзначаны файл.\n\
-help\t\t\t\tнадрукаваць гэты тэкст ды выйсьці.\n\n
14 увага: немагчыма прызначыць пераменную асяродку 'DISPLAY'

$set 20

1 Вышэй шчыліны
2 Ніжэй
3 Сталец
4 Шчыліна
5 Нармальная
6 Вышэй


