$ codeset=utf8

$set 1 #Align

1 Κάτω Κέντρο
2 Κάτω Αριστερά
3 Κάτω Δεξιά
4 Οριζόντια
5 Αριστερά
6 Αριστερά Κάτω
7 Αριστερά Κέντρο
8 Αριστερά Πάνω
9 Σχετική
10 Δεξιά
11 Δεξιά Κάτω
12 Δεξιά Κέντρο
13 Δεξιά Πάνω
14 Πάνω Κέντρο
15 Πάνω Αριστερά
16 Πάνω Δεξιά
17 Κατακόρυφη

$set 2 #BaseDisplay

1 Aborting... dumping core\n
2 Τερματισμός\n
3 %s:      σήμα %d ανιχνεύθηκε\n

$set 3 #Common

1 Άλφα
2 Αυτόματη Απόκρυψη
3 Μεταγλωττίστηκε
4 Επιλογές μεταγλώττισης
5 Μεταγλωττιστής
6 Έκδοση μεταγλωττιστή
7        init
8        keys
9         nls
10        menu
20  windowmenu
11       style
12 Defaults
13 απενεργοποιημένη
14 Σφάλμα
15 Έκδοση Fluxbox
16 Υπερκάλυψη κατά τη μεγιστοποίηση
18 Ορατή

$set 4 #Configmenu

1 Αντιταύτιση
2 Αυτόματη Ανύψωση
3 Κλικ για Ανύψωση
4 Κλικ για Εστίαση
5 Διακόσμησε Προσωρινά Παράθυρα
6 Αλλαγή Επιφάνειας Εργασίας με Ροδέλα Ποντικιού
7 Εστίαση Παραθύρου στην Αλλαγή Επιφάνειας Εργασίας
8 Μοντέλο Εστίασης
9 Εστίασε σε Νέα Παράθυρα
10 Πλήρης Μεγιστοποίηση
11 Image Dithering
12 Opaque Window Moving
14 Mouse Focus
15 Workspace Warping
16 Επιβολή Ψευδο-Διαφάνειας
17 Μενού Άλφα
18 Διαφάνεια
19 Άλφα Εστιασμένου Παραθύρου
20 Άλφα Μη Εστιασμένου Παραθύρου

$set 5 #Ewmh

1 Fatal: Out of memory, can't allocate for EWMH client list

$set 6 #FbTkError

1 Προειδοποίηση! Ύψος > 3200 θέτω Ύψος = 3200
2 Προειδοποίηση! Πλάτος > 3200 θέτω Πλάτος = 3200
3 Προειδοποίηση: δεν μπορώ να φορτώσω την εφεδρική γραμματοσειρά
4 Σφάλμα δέσμευσης.
5 Σφάλμα κατα τη δημιουργία pixmap
6 Δεν μπορώ να δημιουργήσω XImage
7 Σφάλμα ανάγνωσης
8 Προειδοποίηση: Αποτυχία εύρεσης έγκυρου format για άλφα.
9 Προειδοποίηση: Αποτυχία δημιουργίας άλφα εικόνας.
10 Προειδοποίηση: Αποτυχία δημιουργίας άλφα pixmap.
11 Failed to find format for screen(%d)
12 Έλλειψη μνήμης
13 Έλλειψη μνήμης κατά τη δέσμευση blue buffer.
14 Έλλειψη μνήμης κατά τη δέσμευση green buffer.
15 Έλλειψη μνήμης κατά τη δέσμευση red buffer.
16 Αποτυχία ανάγνωσης αντικειμένου θέματος
17 unsupported visual
18 Setting default value

$set 7 #Fluxbox

1 Το όνομα του rc αρχείου είναι άκυρο!
2 Αποτυχία φόρτωσης αρχείου group
3 Αποτυχία φόρτωσης βάσης δεδομένων
4 Retrying with
5 Warning! Could not find screen to map window on!
6 Δεν μπορώ να δημιουργήσω τον κατάλογο %s
7 Δεν βρέθηκαν οθόνες για διαχείριση.\nΣιγουρευτείτε ότι δεν έχετε άλλο διαχειριστή παραθύρων να τρέχει.
8 Error parsing regular expression
9 Fatal! There can only one instance of fluxbox class.
10 Αποτυχία σύνδεσης με τον εξυπηρετητή των X.\nΣιγουρευτείτε ότι ξεκινήσατε τα X πριν ξεκινήσετε το Fluxbox.
11 Προειδοποίηση: Ο εξυπηρετητής των X δεν υποστηρίζει locale
12 Warning: cannot set locale modifiers

$set 8 #Gnome

1 Fatal: Out of memory, can't allocate for GNOME client list

$set 9 #Keys

1 Keys: Σφάλμα στη γραμμή
2 Keys: Failed to merge keytree!
3 Keys: Invalid key/modifier on line

$set 10 #Menu

1 Ρυθμίσεις
2 Εξ ορισμού μενού του
3 Έξοδος
4 Εικονίδια
5 Επίπεδο...
6 On Head...
7 Τοποθέτηση
8 Επαναφόρτωση Ρυθμίσεων
9 Επανεκκίνηση

$set 11 #Remember

1 Διακοσμήσεις
2 Διαστάσεις
3 Μεταφορά στην επιφάνεια εργασίας
4 Επίπεδο
5 Θυμήσου...
6 Τοποθεσία
7 Αποθήκευση στο κλείσιμο
8 Σκιασμένο
9 Sticky
10 Unknown apps key
11 Επιφάνεια εργασίας
12 Head

$set 12 #Screen

1 BScreen::BScreen: an error occured while querying the X server.\n	another window manager already running on display
2 W: %4d x H: %4d
3 BScreen::BScreen: managing screen %d using visual 0x%lx, depth %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 Πελάτες
2 Ανακύκλωσε προς τα πάνω
3 Ανακύκλωσε προς τα κάτω
4 Κατεύθυνση του Slit
5 Επίπεδο του Slit
6 Slit on Head
7 Τοποθέτηση του Slit
8 Slit
9 Αποθήκευση SlitList

$set 14 #Toolbar

1 Διόρθωση ονόματος τρέχουσας επιφάνειας εργασίας
2 Κατάσταση της Iconbar
3 Όλα τα παράθυρα
4 Εικονίδια
5 Καμία
6 Επιφάνεια εργασίας
7 Εικονίδια επιφάνειας εργασίας
8 Επίπεδο της Toolbar
9 Toolbar on Head
10 Τοποθέτηση της Toolbar
11 Toolbar
12 Ποσοστό πλάτους Toolbar
13 Ρολόι: 24Ω
14 Ρολόι: 12Ω
15 Ρύθμιση μορφής ρολογιού
16 Εμφάνιση εικόνων
17 Όχι εικονίδια
18 Επιφάνεια εργασίας χωρίς εικονίδια

$set 15 #Window

1 Χωρίς όνομα

$set 16 #Windowmenu

1 Κλείσιμο
2 Iconify
3 Επίπεδο
4 Βύθιση
5 Μεγιστοποίηση
6 Ανύψωση
7 Αποστολή στο ...
8 Σκίαση
9 Stick
10 Σκότωμα

$set 17 #Workspace

1 Επιφάνεια Εργασίας %d
2 Επιφάνειες Εργασίας
3 Νέα επιφάνεια εργασίας
4 Διαγραφή τελευταίας

$set 18 #fbsetroot

1 Error: must specify one of: -solid, -mod, -gradient\n
2 Couldn't create pixmap atoms, giving up!
3 -display <string>        display connection\n\
-mod <x> <y>             modula pattern\n\
-foreground, -fg <color> modula foreground color\n\
-background, -bg <color> modula background color\n\n\
-gradient <texture>      gradient texture\n\
-from <color>            gradient start color\n\
-to <color>              gradient end color\n\n\
-solid <color>           solid color\n\n\
-help                    print this help text and exit\n

$set 19 #main

1 σφάλμα: το '-display' χρειάζεται ένα όρισμα
2 Bad Alloc
3 Bad cast
4 Εκτός ορίων
5 Σφάλμα κατά την εκτέλεση
6 Standard Exception
7 Άγνωστο σφάλμα
8 σφάλμα: το '-log' χρειάζεται ένα όρισμα
9 Αρχείο Καταγραφής
10 Γίνεται καταγραφή στο
11 σφάλμα: το '-rc' χρειάζεται ένα όρισμα
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tχρησιμοποίησε σύνδεση display.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit\n\
-no-toolbar\t\t\tdo not provide a toolbar\n\
-rc <string>\t\t\tχρησιμοποίησε εναλλακτικό αρχείο πόρων.\n\
-version\t\t\tεμφάνιση έκδοσης και τερματισμός.\n\
-info\t\t\t\tεμφάνιση μερικών χρήσιμων πληροφοριών.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tαποθήκευση log σε αρχείο.\n\
-help\t\t\t\tεμφάνιση αυτού του κειμένου βοηθείας και τερματισμός.\n\n
14 προειδοποίηση: δεν είναι δυνατός ο ορισμός της μεταβλητής περιβάλλοντος 'DISPLAY'

$set 20

1 Πάνω απο το Dock
2 Πάτος
3 Desktop
4 Dock
5 Κανονική
6 Κορυφή


