$ codeset=UTF-8

$set 1 #Align

1 底部中間
2 底部左側
3 底部右側
4 水平
5 左側
6 左側底部
7 左側中間
8 左側頂端
9 相對的
10 右側
11 右側底部
12 右側中間
13 右側頂端
14 頂端中間
15 頂端左側
16 頂端右側
17 垂直

$set 2 #BaseDisplay

1 正在中斷…傾印核心\n
2 關機中\n
3 %s：     擷取到信號 %d\n

$set 3 #Common

1 透明度(Alpha)
2 自動隱藏
3 編譯
4 編譯的選項
5 編譯器
6 編譯器版本
7        init
8        keys
9         nls
10        menu
20  windowmenu
11       style
12 預設
13 停用
14 錯誤
15 Fluxbox 版本
16 被最大化視窗覆蓋
17 GIT 修訂
18 可見的
19 在此樣式中沒有指定任何背景選項。\n請查看手冊或閱讀 FAQ。

$set 4 #Configmenu

1 平滑字形
2 自動提升視窗
3 單擊提升視窗
4 單擊指定焦點
5 裝飾暫時性視窗
6 滑鼠滾輪切換桌面
7 工作區變更時指定焦點視窗
8 焦點指定型式
9 新視窗指定焦點
10 視窗完全最大化
11 影像遞色
12 移動時混濁視窗
14 焦點跟隨滑鼠
15 視窗滑移工作區邊界
16 強制假性透明
17 選單透明度
18 透明度
19 焦點視窗透明度
20 非焦點視窗透明度
21 分頁選項
22 標題欄中的分頁
23 外部分頁寬度
24 單擊指定分頁焦點
25 滑鼠指定分頁焦點

$set 5 #Ewmh

1 嚴重：記憶體不足，無法配置給 EWMH 客戶端清單

$set 6 #FbTkError

1 警告！高度>3200, 設定高度=3200
2 警告！寬度>3200, 設定寬度=3200
3 警告：無法載入權宜字型
4 配額錯誤。
5 建立像素對映時發生錯誤
6 無法建立 XImage
7 讀取時失敗
8 警告：尋找有效透明格式時失敗。
9 警告：建立透明圖片時失敗。
10 警告：建立透明像素對映時失敗。
11 尋找螢幕規格 (%d) 時失敗
12 記憶體不足
13 配置藍色緩衝區時記憶體不足。
14 配置綠色緩衝區時記憶體不足。
15 配置紅色緩衝區時記憶體不足。
16 讀取佈景主題項目時失敗
17 視覺不支援
18 設定預設值

$set 7 #Fluxbox

1 rc 檔案名稱無效！
2 載入 groupfile 時失敗
3 載入 database 時失敗
4 重新嘗試時以
5 警告！找不到可以顯示視窗的螢幕！
6 無法建立目錄 %s
7 找不到可以管理的螢幕。\n確定您沒有正在執行其他視窗管理員。
8 剖析正規表示式時發生錯誤
9 嚴重！只能有一個實體的 fluxbox 類別。
10 無法連線到 X 伺服器。\n確定您啟動 Fluxbox 之前已啟動 X。
11 警告：X 伺服器不支援語區
12 警告：無法設定語區

$set 8 #Gnome

1 嚴重：記憶體不足，無法配置給 GNOME 客戶端清單

$set 9 #Keys

1 按鍵：錯誤發生於列號
2 按鍵：合併 keytree 時失敗！
3 按鍵：無效的按鍵於列號

$set 10 #Menu

1 配置
2 Fluxbox 預設選單
3 離開
4 圖示
5 圖層…
6 在顯示器上…
7 放置位址
8 重新載入配置
9 重新啟動
10 警告：不正常的 [編碼] 標記

$set 11 #Remember

1 裝飾
2 維度
3 跳至工作區
4 圖層
5 記住…
6 位置
7 關閉時儲存
8 陰影
9 固定
10 不明的應用軟體鍵
11 工作區
12 顯示器

$set 12 #Screen

1 BScreen::BScreen：向 X 伺服器查詢時發生錯誤。\n	 螢幕中已經執行另外的視窗管理員
2 W: %4d x H: %4d
3 BScreen::BScreen：管理螢幕 %d 是使用視訊 0x%lx、色深 %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 客戶端
2 向下循環
3 向上循環
4 工具條方向
5 工具條圖層
6 工具條於顯示器上
7 工具條放置位址
8 工具條
9 儲存工具條清單

$set 14 #Toolbar

1 重新命名目前工作區
2 圖示欄模式
3 所有視窗
4 圖示
5 無
6 工作區
7 工作區圖示
8 工具列圖層
9 工具列於顯示器上
10 工具列放置位址
11 工具列
12 工具列寬度百分比
13 時鐘：24 小時制
14 時鐘：12 小時制
15 編輯時鐘格式
16 顯示圖片
17 無圖示
18 工作區無圖示

$set 15 #Window

1 未命名

$set 16 #Windowmenu

1 關閉
2 最小化
3 圖層
4 降低
5 最大化
6 提升
7 傳送到…
8 陰影
9 固定
10 砍掉

$set 17 #Workspace

1 工作區 %d
2 工作區
3 新增工作區
4 移除末端工作區

$set 18 #fbsetroot

1 錯誤：必須指定下列之一：-solid、-mod、-gradient\n
2 無法建立像素對映元素，放棄！
3 -display <string>       顯示連接\n\
-mod <x> <y>             模塊式樣\n\
-foreground, -fg <color> 模塊前景顏色\n\
-background, -bg <color> 模塊背景顏色\n\n\
-gradient <texture>      梯度紋理\n\
-from <color>            梯度開始顏色\n\
-to <color>              梯度結束顏色\n\n\
-solid <color>           單色\n\n\
-help                    印出此說明文字然後離開\n

$set 19 #main

1 錯誤：「-display」需要一個引數
2 不當的記憶體配置
3 不當的類型轉換
4 超出範圍
5 執行階段錯誤
6 標準異常
7 不明的錯誤
8 錯誤：「-log」需要引數
9 記錄檔
10 登入記錄到
11 錯誤：「-rc」需要一個引數
12 錯誤：「-screen」需要一個引數
13 Fluxbox %s：(c) %s Fluxbox Team\n\
網站：http://www.fluxbox.org/\n\n\
-display <string>\t\t所使用的顯示連接。\n\
-screen <all|int,int,int>\t只運行於指定的螢幕。\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\t使用替代的資源檔案。\n\
-version\t\t\t顯示版本然後離開。\n\
-info\t\t\t\t顯示一些有用資訊。\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\t記錄輸出到檔案。\n\
-help\t\t\t\t顯示此說明文字然後離開。\n\n
14 警告：無法設定環境變數「DISPLAY」

$set 20 #layers

1 停駐區之上
2 底部
3 桌面
4 停駐區
5 一般
6 頂端
