$ codeset=UTF-8

$set 1 #Align

1 Nere i mitten
2 Nere till vänster
3 Nere till höger
4 Horisontell
5 Till vänster
6 Till vänster nere
7 Till vänster i mitten
8 Till vänster uppe
9 Relativ
10 Till höger
11 Till höger nere
12 Till höger i mitten
13 Till höger uppe
14 Uppe i mitten
15 Uppe till vänster
16 Uppe till höger
17 Vertikal

$set 2 #BaseDisplay

1 Avbryter... dumpar core\n
2 Stänger ner\n
3 %s:    signal %d fångad\n

$set 3 #Common

1 Alpha
2 Göm automatiskt
3 Kompilerad
4 Kompilerings val
5 Kompilator
6 Kompilator version
13 avstängd
14 Fel
16 Maximera över
18 Synbar
19 Det finns ingen bakgrunds val för detta tema. Var vänlig läs manualen eller läs FAQ.

$set 4 #Configmenu

1 Kantutjämning
2 Höj automatiskt
3 Klick höjer fönstret
4 Klicka för fokus
5 Dekorera transienta fönster
7 Fokusera fönster vid skrivbordsbyte
8 Fokusmodell
9 Fokus på nya fönster
10 Full maximering
11 Bilddithering
12 Ogenomskinlig fönsterförflyttning
13 Halv hafsig fokus
14 Hafsig fokus
15 Dra fönster mellan arbetsytor
16 Tvinga pseudo-transparens
17 Meny alpha
18 Transparens
19 Fokuserad fönsters alpha
20 Icke fokuserad fönsters alpha
21 Flikval
22 Flikar i titelraden
23 Externa flikars bredd
24 Klick i flik ger fokus
25 Musfokus i flik

$set 5 #Ewmh

1 Slut på minne, kan inte allokera minne för EWMH klientlista. 

$set 6 #FbTkError

1 Varning! Höjden överskrider 3200, sätter höjd till 3200.
2 Varning! Bredden överskrider 3200, sätter bredden till 3200
3 Varning! Kan inte ladda grundfonten
4 Allokerings fel.
5 Fel vid skapande av pixmap.
6 Kan inte skapa XImage
7 Kunde inte läsa
8 Varning! Kunde inte hitta rätt format för alpha.
9 Varning! Kunde inte skapa alpha bild.
10 Varning! Kunde inte skapa alpha pixmap.
11 Misslyckades att hitta format för skärm(%d)
12 Slut på minne
13 Slut på minne vid allokering av blå buffer.
14 Slut på minne vid allokering av grön buffer.
15 Slut på minne vid allokering av röd buffer.
16 Misslyckades att läsa tema objekt
18 Sätter grundvärde


$set 7 #Fluxbox

1 rc filnamn är felaktigt!
2 Kunde inte ladda gruppfil
3 Kunde inte ladda databasen
4 Försöker igen med
5 Varning! Kunde inte hitta skärm att placera fönster på
6 kan inte skapa katalog %s
7 Kan inte hitta skärm att hantera.\nSe till att det inte är någon annan fönsterhanterare som körs.
8 Fel vid evalueringen av regular expression
10 Kan inte ansluta till X server.\nStarta X innan du startar Fluxbox.
11 Varning! X server har ingen support för locale
12 Varning! Kan inte sätta locale modifiers

$set 8 #Gnome

1 Fatal: Slut på minne. Kan inte allokera GNOME klientlista.

$set 9 #Keys

1 Keys: Fel på rad
2 Keys: Misslyckades att slå ihop keytree!
3 Keys: Felaktig tangent/modifierare på rad 

$set 10 #Menu

1 Konfiguration
2 Fluxbox grundmeny
3 Avsluta
4 Ikoner
5 Lager...
6 På huvud...
7 Placering
8 Ladda om konfiguration
9 Starta om
10 Varning: obalanserade [encoding] taggar

$set 11 #Remember

1 Dekorationer
2 Dimensioner
3 Hoppa till arbetsyta
4 Lager
5 Kom ihåg...
6 Position
7 Spara vid stängning
8 Skugga
9 Klibbig
10 Okänd apps tag
11 Arbetsyta
12 Huvud
13 Transparens


$set 12 #Screen

1 BScreen::BScreen: Ett fel uppstod med X.\nEn annan fönsterhanterare körs.
2 W: %4d x H: %4d
3 BScreen::BScreen: Hanterare skärm %d med visual 0x%lx, djup %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 Klienter
2 Flytta ner
3 Flytta upp
4 Slitriktning
5 Slitlager
6 Slit på huvud
7 Slitplacering
8 Slit
9 Spara slitlistan

$set 14 #Toolbar

1 Ändra namn på skrivbordet
2 Ikonradsläge
3 Alla fönster
4 Ikoner
5 Inga
6 Arbetsyta
7 Arbetsytans ikoner
9 Verktygsrads lager
10 Verktygslistens placering
11 Verktygslist
12 Verktygsradens bredd i procent
13 Klocka i 24h format
14 Klocka i 12h format
15 Ändra klockformat
16 Visa bilder
17 Inga ikoner
18 Inga ikoner för arbetsytan

$set 15 #Window

1 Inget namn

$set 16 #Windowmenu

1 Stäng
2 Ikonifiera
4 Sänk
5 Maximera
6 Höj
7 Skicka till...
8 Skugga
9 Klibbig
10 Döda
11 Använd grundinställningar

$set 17 #Workspace

1 Skrivbord %d
2 Skrivbord
3 Nytt skrivbord
4 Ta bort sista

$set 18 #fbsetroot

1 fel: måste specifiera en av: -solid, -mod, -gradient\n
3 -display <string>        skärmanslutning\n\
-mod <x> <y>             modulamönster\n\
-foreground, -fg <color> förgrundsfärg för modulamönster\n\
-background, -bg <color> bakgrundsfärg för modulamönster\n\n\
-gradient <texture>      texturlutning\n\
-from <color>            startfärg\n\
-to <color>              slutfärg\n\n\
-solid <color>           solid färg\n\n\
-help                    visa denna hjälptext och avsluta\n

$set 19 #main

1 fel: '-display' kräver ett argument
11 fel: '-rc' kräver ett argument
13 Fluxbox %s: (c) %s Fluxbox Team\n\n\
-display <sträng>\t\tanvänd skärmanslutning.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <sträng>\t\t\tanvänd alternativ resursfil.\n\
-version\t\t\tvisa version och avsluta.\n\
-info\t\t\t\tVisar nyttig information.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filenamn>\t\t\tloggar utskriften till en fil.\n\
-help\t\t\t\tvisa denna hjälptext och avsluta.\n\n

$set 20 #layers

1 Över dock
2 Längst ner
3 Arbetsyta
4 Dock
5 Normal
6 Top
