$ codeset=UTF-8

$set 1 #Align

1 下方居中
2 左下方
3 右下方
4 水平
5 靠左
6 左下方
7 左侧居中
8 左上方
9 扩展
10 靠右
11 右下方
12 右侧居中
13 右上方
14 上方居中
15 左上方
16 左上方
17 垂直

$set 2 #BaseDisplay

1 退出... 存储崩溃文件\n
2 关闭\n
3 %s:      捕获信号 %d\n

$set 3 #Common

1 Alpha值
2 自动隐藏
3 编译时间
4 编译选项
5 编译器
6 编译器版本
7 	初始化
8 	键绑订
9 	本地语言支持
10 	菜单
11 	样式
12 默认配置
13 禁用
14 错误
15 Fluxbox版本
16 最大化时盖过
17 GIT修订版本号
18 可见
19 该主题未定义任何背景。\n请参考手册或阅读FAQ。

$set 4 #Configmenu

1 反锯齿
2 自动提升
3 点击提升
4 点击聚焦
5 装饰临时窗口
6 鼠标滚轮切换工作区
7 Focus Window on Workspace Change
8 聚焦模式
9 聚焦新窗口
10 完全最大化
11 图像仿色
12 移动时显示窗口内容
14 鼠标聚焦
15 工作区回旋
16 强制伪透明
17 菜单Alpha值
18 透明
19 聚焦窗口Alpha值
20 非聚焦窗口Alpha值
21 标签选项
22 标签置于标题栏内
23 外置标签宽度
24 点击聚焦标签
25 鼠标聚焦标签

$set 5 #Ewmh

1 致命错误: 内存不足,无法为EWMH client list分配内存

$set 6 #FbTkError

1 警告！ Height > 3200 ，设置 Height = 3200
2 警告！ Width > 3200 ，设置 Width = 3200
3 警告: 不能加载备用字体
4 分配错误。
5 创建像素映射出错
6 无法创建XImage
7 读取失败
8 警告：无法为alpha找到有效格式。
9 警告：创建alpha图片失败。
10 警告：创建alpha像素映射失败。
11 查找屏幕格式失败(%d)
12 内存不足
13 内存不足，无法分配给blue buffer。
14 内存不足，无法分配给green buffer。
15 内存不足，无法分配给red buffer。
16 读取主题项目失败
17 不支持图像模式
18 设置默认值

$set 7 #Fluxbox

1 无效的rc文件名!
2 加载组文件失败
3 加载数据库失败
4 Retrying with
5 警告！无法找到屏幕来绘制窗口！
6 无法创建 %s 目录
7 无法找到需要控制的屏幕。\n请确保没有其他窗口管理器正在运行。
8 语法规则解析错误
9 致命错误！fluxbox类只能有一个实例。
10 无法连接X server。\n请确保你开启Fluxbox之前开启了X。
11 警告：X server不支持locale
12 警告：无法设置locale modifiers

$set 8 #Gnome

1 错误: 内存不足, 无法为GNOME client list分配内存

$set 9 #Keys

1 Keys: Error on line
2 Keys: Failed to merge keytree!
3 Keys: Invalid key/modifier on line

$set 10 #Menu

1 配置
2 Fluxbox默认菜单
3 退出
4 图标
5 层次...
6 位于监视器...
7 位置
8 重新加载配置文件
9 重启Fluxbox
10 警告：[encoding]标签不对称

$set 11 #Remember

1 装饰
2 尺寸
3 跳至工作区
4 层
5 记住...
6 位置
7 关闭时保存
8 卷起
9 粘住
10 未知按键
11 工作区
12 监视器

$set 12 #Screen

1 BScreen::BScreen: 询问X server时发生错误.\n 另外一个窗口管理器正运行,display为
2 W: %4d x H: %4d
3 BScreen::BScreen: 控制屏幕 %d 使用图像模式 0x%lx, 色彩深度 %d\n
4 W: %04d x H: %04d

$set 13 #Slit

1 Clients
2 下卷
3 上卷
4 Slit方向
5 Slit层次
6 Slit位于监视器
7 Slit位置
8 Slit
9 保存Slit列表

$set 14 #Toolbar

1 重命名当前工作区
2 图标栏模式
3 所有窗口
4 所有最小化的窗口
5 无
6 当前工作区所有窗口
7 当前工作区所有最小化的窗口
8 工具栏层次
9 工具栏位于监视器
10 工具栏位置
11 工具栏
12 工具栏宽度百分比
13 时钟：24小时格式
14 时钟：12小时格式
15 编辑时间显示格式
16 显示图标
17 所有未最小化的窗口
18 当前工作区所有未最小化的窗口

$set 15 #Window

1 未命名

$set 16 #Windowmenu

1 关闭
2 最小化
3 层次
4 降低
5 最大化
6 提升
7 发送到...
8 卷起
9 粘住
10 杀死

$set 17 #Workspace

1 工作区 %d
2 工作区
3 新建工作区
4 移除最后一个工作区

$set 18 #fbsetroot

1 错误: 必须指定: -solid, -mod, -gradient 三个参数中的一个.\n
2 无法创建pixmap atoms，放弃！
3 -display <string>        display connection\n\
-mod <x> <y>             modula pattern\n\
-foreground, -fg <color> modula foreground color\n\
-background, -bg <color> modula background color\n\n\
-gradient <texture>      gradient texture\n\
-from <color>            gradient start color\n\
-to <color>              gradient end color\n\n\
-solid <color>           solid color\n\n\
-help                    显示帮助信息\n

$set 19 #main

1 错误: '-display'需要一个参数
2 Bad Alloc
3 Bad cast
4 超出显示范围
5 运行时错误
6 标准例外
7 未知错误
8 错误: '-log'需要一个参数
9 日志文件
10 记录日志到
11 错误: '-rc'需要一个参数
12 错误: '-screen'需要一个参数
13 Fluxbox %s: (c) %s Fluxbox Team\n\
主页: http://www.fluxbox.org/\n\n\
-display <string>\t\t指定display连接。\n\
-screen <all|int,int,int>\t在特定的屏幕上显示。\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\t使用指定的资源文件。\n\
-version\t\t\t显示版本号然后退出。\n\
-info\t\t\t\t显示一些有用的信息。\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\t将输出记录到文件中。\n\
-help\t\t\t\t显示这则帮助信息然后退出。\n\n
14 警告: 无法设置环境变量'DISPLAY'

$set 20

1 在Dock之上
2 底部
3 桌面
4 Dock
5 普通
6 顶部


