$ codeset=UTF-8

$set 1 #Align

1 Embaixo no centro
2 Embaixo à esquerda
3 Embaixo à direita
4 Horizontal
5 À esquerda
6 À esquerda embaixo
7 À esquerda no centro
8 À esquerda em cima
9 Relativo
10 À direita
11 À direita embaixo
12 À direita no centro
13 À direita em cima
14 Em cima no centro
15 Em cima à esquerda
16 Em cima à direita
17 Vertical
18 No centro
19 Em cima
20 Embaixo

$set 2 #BaseDisplay

1 Abortando... despejando o núcleo\n
2 Desligando\n
3 %s:      sinal %d recebido\n

$set 3 #Common

1 Alfa
2 Auto-ocultar
3 Compilado
4 Opções de compilação
5 Compilador
6 Versão do compilador
7        init
8        keys
9         nls
10        menu
20  windowmenu
11       style
12 Padrões
13 desativado
14 Erro
15 Versão do Fluxbox
16 Maximizar sobre
17 Revisão do GIT
18 Visível
19 Não há uma opção de fundo especificada neste tema.\nPor favor, consulte o manual ou leia o FAQ.

$set 4 #Configmenu

1 Antialias
2 Por cima automaticamente
3 Clicar põe por cima
4 Clicar para focar
5 Decorar janelas transitórias
6 Roda do mouse sobre o desktop muda espaço de trabalho
7 Focar a janela ao mudar espaço de trabalho
8 Modelo de foco
9 Focar novas janelas
10 Maximização total
11 Ditherização de imagens
12 Movimentação opaca de janelas
14 Foco com o mouse
15 Espaços de trabalho contínuos
16 Forçar pseudotransparência
17 Alfa do Menu
18 Transparência
19 Alfa da janela com foco
20 Alfa da janela sem foco
21 Opções de abas
22 Abas na barra de título
23 Largura da aba externa
24 Clicar foca a aba
25 Mouse foca a aba
26 Opções de maximização
27 Ignorar incremento do redimensionamento
28 Desabilitar a movimentação
29 Desabilitar o redimensionamento

$set 5 #Ewmh

1 Fatal: memória insuficiente, não é possível alocar memória para a lista de clientes EWMH

$set 6 #FbTkError

1 Atenção! Altura > 3200, definindo Altura = 3200
2 Atenção! Largura > 3200, definindo Largura = 3200
3 Aviso: não é possível carregar a fonte de reserva
4 Erro de alocação.
5 Erro na criação de pixmap
6 Não é possível criar XImage
7 Falha na leitura
8 Aviso: não foi possível encontrar um formato válido para o alfa.
9 Aviso: não foi possível criar uma imagem alfa.
10 Aviso: não foi possível criar um pixmap alfa.
11 Não foi possível encontrar um formato para a tela(%d)
12 Memória insuficiente
13 Memória insuficiente ao alocar o buffer do azul.
14 Memória insuficiente ao alocar o buffer do verde.
15 Memória insuficiente ao alocar o buffer do vermelho.
16 Falha na leitura de item do tema
17 visual não suportado
18 Definindo pelo valor padrão

$set 7 #Fluxbox

1 O nome do arquivo rc é inválido!
2 Não foi possível carregar o arquivo de grupos
3 Não foi possível carregar a base de dados
4 Tentando novamente com
5 Atenção! Não foi encontrada uma tela para onde mapear a janela!
6 Não é possível criar diretorio %s
7 Não foram encontradas telas para gerenciar.\nAssegure-se de que não há outro gerenciador de janelas ativo.
8 Erro ao analisar expressão regular
9 Fatal! Só é possível haver uma instância da classe fluxbox.
10 Não é possível conectar-se ao servidor X.\nAssegure-se de que o X foi iniciado antes do Fluxbox.
11 Aviso: o servidor X não suporta o locale
12 Aviso: não é possível estabelecer modificadores de locale

$set 8 #Gnome

1 Fatal: memória insuficiente, não é possível alocar memória para a lista de clientes do GNOME

$set 9 #Keys

1 Keys: erro na linha
2 Keys: não foi possível fundir a árvore de teclas!
3 Keys: tecla/modificador inválido na linha

$set 10 #Menu

1 Configuração
2 Menu padrão do Fluxbox
3 Sair
4 Janelas minimizadas
5 Camada...
6 No monitor...
7 Posicionamento
8 Recarregar a configuração
9 Reiniciar
10 Aviso: etiquetas [codificação] desbalanceadas

$set 11 #Remember

1 Decorações
2 Dimensões
3 Ir para o espaço de trabalho
4 Camada
5 Memorizar...
6 Posição
7 Salvar ao sair
8 Enrolada
9 Pregada
10 Chave apps desconhecida
11 Espaço de trabalho
12 Monitor
13 Transparência

$set 12 #Screen

1 BScreen::BScreen: ocorreu um erro ao interrogar o servidor X.\n outro gerenciador de janelas está ativo
2 L: %4d x A: %4d
3 BScreen::BScreen: gerenciando tela %d usando visual 0x%lx, com profundidade %d\n
4 L: %04d x A: %04d

$set 13 #Slit

1 Clientes
2 Circular para baixo
3 Circular para cima
4 Direção do Slit:
5 Camada do Slit
6 Slit no monitor
7 Posicionamento do Slit
8 Slit
9 Salvar lista do Slit

$set 14 #Toolbar

1 Renomear o espaço de trabalho atual
2 Modo da barra de janelas
3 Todas as janelas
4 Minimizadas
5 Nenhuma
6 Espaço de trabalho
7 Minimizadas no espaço de trabalho
8 Camada da barra de tarefas
9 Barra de tarefas no monitor
10 Posicionamento da barra de tarefas
11 Barra de tarefas
12 Largura da barra em porcentagem
13 Relógio: 24h
14 Relógio: 12h
15 Editar o formato do relógio
16 Exibir imagens
17 Não minimizadas
18 Não minimizadas no espaço de trabalho

$set 15 #Window

1 Sem nome

$set 16 #Windowmenu

1 Fechar
2 Minimizar
3 Camada
4 Por baixo
5 Maximizar
6 Por cima
7 Enviar para...
8 Enrolar
9 Pregar
10 Matar
11 Usar valores padrão
12 Definir o título da janela

$set 17 #Workspace

1 Espaço de trabalho %d
2 Espaços de trabalho
3 Novo espaço de trabalho
4 Remover o último

$set 18 #fbsetroot

1 Erro: é necessário especificar um entre: -solid, -mod, -gradient\n
2 Não foi possível criar átomos de pixmap, desistindo!
3 -display <string>        conexão com monitor\n\
-mod <x> <y>             padrão modula\n\
-foreground, -fg <cor>   cor de primeiro plano modula\n\
-background, -bg <cor>   cor de fundo modula\n\n\
-gradient <textura>      textura do gradiente\n\
-from <cor>              cor de início do gradiente\n\
-to <cor>                cor de término do gradiente\n\n\
-solid <cor>             cor sólida\n\n\
-help                    exibir esta ajuda e sair\n

$set 19 #main

1 erro: '-display' requer um argumento
2 Alocação inválida
3 Conversão inválida
4 Fora dos limites
5 Erro de execução
6 Exceção padrão
7 Erro desconhecido
8 erro: '-log' requer um argumento
9 Arquivo de log
10 Registrando o log em
11 erro: '-rc' requer um argumento
12 erro: '-screen' requer um argumento
13 Fluxbox %s: (c) %s Fluxbox Team\n\
Site: http://www.fluxbox.org/\n\n\
-display <string>\t\tusar conexão com o monitor.\n\
-screen <all|int,int,int>\texecutar apenas nas telas especificadas.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tusar arquivo de configuração alternativo.\n\
-version\t\t\texibir a versão e finalizar.\n\
-info\t\t\t\texibir algumas informações úteis.\n\
-list-commands\t\t\tlistar todos os comandos de teclas válidos.\n\
-sync\t\t\t\tsincronizar com o servidor X para depuração.\n\
-log <nomedoarquivo>\t\t\tlogar a saída no arquivo.\n\
-help\t\t\t\texibir esta ajuda e sair.\n\n
14 aviso: não foi possível definir a variável de ambiente 'DISPLAY'

$set 20 #layers

1 Acima do dock
2 Embaixo
3 Desktop
4 Dock
5 Normal
6 Em cima



