$ codeset=ISO-8859-2

$set 1 #Align

1 Dole v strede
2 Dole vľavo
3 Dole vpravo
4 Horizontálne
5 Vľavo
6 Vľavo dole
7 Vľavo v strede
8 Vľavo hore
9 Relatívne
10 Vpravo
11 Vpravo dole
12 Vpravo v strede
13 Vpravo hore
14 Hore v strede
15 Hore vľavo
16 Hore vpravo
17 Vertikálne

$set 2 #BaseDisplay

1 Ukončujem... zaznamenávam pád aplikácie\n
2 Vypínam\n
3 %s:      zachytený signál %d\n

$set 3 #Common

1 Alfa
2 Automatické skrývanie
3 Skompilované
4 Nastavenie kompilácie
5 Kompilátor
6 Verzia kompilátora
7        init
8     klávesy
9         nls
10        menu
20  windowmenu
11        štýl
12 Predvolené
13 vypnuté
14 Chyba
15 Verzia Fluxboxu
16 Maximalizovať nad všetkými oknami
17 Revízia GIT
18 Viditeľné
19 V tomto štýle nie je uvedené žiadne pozadie.\nPrezrite si prosím manuál alebo prečítajte Často kladené otázky.

$set 4 #Configmenu

1 AntiAlias
2 Automatické zdvihnutie okna
3 Zdvihnutie okna kliknutím
4 Prechod do okna kliknutím
5 Dekorovať dočasné okná
6 Prepínanie plôch kolieskom myši
7 Prechod do okna pri zmene pracovnej plochy
8 Model prechodu do okna
9 Prejsť do nových okien
10 Úplná maximalizácia
11 Dithering obrazu
12 Pohyb okna so zobrazovaním obsahu
14 Zameranie myši
15 Warping pracovnej plochy
16 Vnútiť pseudo-priehľadnosť
17 Alfa menu
18 Priehľadnosť
19 Alfa zameraného okna
20 Alfa nezameraného okna
21 Voľby kariet
22 Karty v titulkovej lište
23 Šírka externej karty
24 Kliknutie zameranie karty
25 Myš zameranie karty

$set 5 #Ewmh

1 Vážna chyba: Došla pamäť, nedá sa alokovať pre zoznam klientov EWMH

$set 6 #FbTkError

1 Varovanie! Výška > 3200, nastavuje sa Výška = 3200
2 Varovanie! Šírka > 3200, nastavuje sa Šírka = 3200
3 Varovanie: nedá sa načítať záložné písmo
4 Chyba alokácie.
5 Chyba pri vytváraní pixmap
6 Nedá sa vytvoriť XImage
7 Zlyhalo čítanie
8 Varovanie: Nepodarilo sa nájsť platný formát pre alfu.
9 Varovanie: Nepodarilo sa vytvoriť alfa obrázok.
10 Varovanie: Nepodarilo sa vytvoriť alfa pixmap.
11 Nepodarilo sa nájsť formát pre obrazovku(%d)
12 Došla pamäť
13 Počas alokácie modrého zásobníka došla pamäť.
14 Počas alokácie zeleného zásobníka došla pamäť.
15 Počas alokácie červeného zásobníka došla pamäť.
16 Nepodarilo sa načítať položku témy
17 nepodporovaný visual
18 Nastavuje sa predvolená hodnota

$set 7 #Fluxbox

1 názov rc súboru je neplatný!
2 Nepodarilo sa načítať skupinový súbor
3 Nepodarilo sa načítať databázu
4 Zkúšam znovu s
5 Varovanie! Nepodarilo sa nájsť obrazovku pre namapovanie tohto okna!
6 Nedá sa vytvoriť adresár %s
7 Nepodarilo sa nájsť obrazovky pre spravovanie.\nUistite sa, že nemáte spusteného iného správcu okien.
8 Chyba pri spracovaní regulárneho výrazu
9 Vážna chyba! Môže existovať iba jedna inštancia trieky fluxbox.
10 Nedá sa pripojiť k X serveru.\nUistite sa, že ste spustili X pred spustením Fluxboxu.
11 Varovanie: X server nepodporuje locale
12 Varovanie: nedajú sa nastaviť modifikátory locale

$set 8 #Gnome

1 Vážna chyba: Došla pamäť, nedá sa alokovať pre zoznam klientov GNOME

$set 9 #Keys

1 Klávesy: Chyba na riadku
2 Klávesy: Nepodarilo sa zlúčiť strom kláves!
3 Klávesy: Neplatný kláves/modifikáror na riadku

$set 10 #Menu

1 Konfigurácia
2 Predvolené menu Fluxboxu
3 Koniec
4 Ikony
5 Vrstva...
6 V hlavičke...
7 Umiestnenie
8 Znovu načítať konfiguráciu
9 Reštartovať
10 Varovanie: nevyvážené [encoding] tagy

$set 11 #Remember

1 Dekorácie
2 Rozmery
3 Prejsť na pracovnú plochu
4 Vrstva
5 Pamätať...
6 Pozícia
7 Uložiť pri zavretí
8 Zrolovať
9 Prilepiť
10 Neznámy kláves aplikácie
11 Pracovná plocha
12 Hlavička
13 Priehľadnosť

$set 12 #Screen

1 BScreen::BScreen: pri dotazovaní X servera sa vyskytla chyba.\n	na displeji je už spustený iný správca okien
2 Š: %4d x V: %4d
3 BScreen::BScreen: spravujem obrazovku %d používam visual 0x%lx, hĺbka %d\n
4 Š: %04d x V: %04d

$set 13 #Slit

1 Klienti
2 Rolovať dole
3 Rolovať hore
4 Smer slitu
5 Vrstva slitu
6 Slit v hlavičke
7 Umiestnenie slitu
8 Slit
9 Uložiť zoznam slitu

$set 14 #Toolbar

1 Premenovať aktuálnu pracovnú plochu
2 Režim lišty ikon
3 Všetky okná
4 Ikony
5 Žiadne
6 Pracovná plocha
7 Ikony pracovnej plochy
8 Vrstva nástrojovej lišty
9 Nástrojová lišta v hlavičke
10 Umiestnenie nástrojovej lišty
11 Nástrojová lišta
12 Šírka nástrojovej lišty v percentách
13 Hodiny: 24h
14 Hodiny: 12h
15 Upraviť formát hodín
16 Zobraziť obrázky
17 Žiadne ikony
18 Pracovná plocha žiadne ikony

$set 15 #Window

1 Nepomenované

$set 16 #Windowmenu

1 Zavrieť
2 Ikonifikovať
3 Vrstva
4 Nižšie
5 Maximalizovať
6 Vyššie
7 Odoslať do ...
8 Zrolovať
9 Prilepiť
10 Zabiť
11 Použiť predvolené

$set 17 #Workspace

1 Pracovná plocha %d
2 Pracovné plochy
3 Nová pracovná plocha
4 Odstrániť poslednú

$set 18 #fbsetroot

1 Chyba: musíte zadať jedno z: -solid, -mod, -gradient\n
2 Nepodarilo sa vytvoriť pixmap atómy, vzdávam to!
3 -display <string>        spojenie na displej\n\
-mod <x> <y>             modula vzor\n\
-foreground, -fg <color> modula farba popredia\n\
-background, -bg <color> modula farba pozadia\n\n\
-gradient <texture>      textúra prechodu\n\
-from <color>            úvodná farba prechodu\n\
-to <color>              záverečná farba prechodu\n\n\
-solid <color>           plná farba\n\n\
-help                    zobraziť tohto textu pomocníka a skončiť\n

$set 19 #main

1 chyba: '-display' vyžaduje argument
2 Zlá alokácia
3 Zlý výber
4 Mimo rozsahu
5 Chyba pri behu
6 Štandardná výnimka
7 Neznáma chyba
8 chyba: '-log' potrebuje argument
9 Súbor so záznamom
10 Zaznamenávam do
11 chyba: '-rc' vyžaduje argument
12 chyba: '-screen' vyžaduje argument
13 Fluxbox %s: (c) %s Fluxbox Team\n\
Webová stránka: http://www.fluxbox.org/\n\n\
-display <string>\t\tpoužiť pripojenie k displeju.\n\
-screen <all|int,int,int>\tspustiť iba na zadaných obrazovkách.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tpoužiť alternatívny súbor zdrojov.\n\
-version\t\t\tzobraziť verziu a skončiť.\n\
-info\t\t\t\tzobraziť nejaké užitočné informácie.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tzaznamenávať výstup do súboru.\n\
-help\t\t\t\tzobraziť tento text pomocníka a skončiť.\n\n
14 varovanie: nepodarilo sa nastaviť premennú prostredia 'DISPLAY'

$set 20 #layers

1 Nad dokom
2 Dole
3 Plocha
4 Dok
5 Normálne
6 Vrch


