$ codeset=UTF-8

$set 1 #Align

1 Долу в Средата
2 Долу в Ляво
3 Долу в Дясно
4 Хоризонтално
6 Долу в Ляво
7 Ляво в Средата
8 Горе в Ляво
11 Долу в Дясно
12 Дясно в Средата
13 Горе в Дясно
14 Горе в Средата
15 Горе в Ляво
16 Горе в Дясно
17 Верткално

$set 2 #BaseDisplay


$set 3 #Common

2 Автоматично скриване

$set 4 #Configmenu

2 Автоматично Повдигане
4 Кликни за Фокус
7 Фокусирай Прозорец при смяна на Работните Места
8 Тип фокус
9 Фокусирай Новите Прозорци
10 Пълно Увеличаване
11 Image Dithering
12 Непрозрачно местене на прозорците
13 Полу Sloppy Фокус
14 Sloppy Фокус
15 Workspace Warping

$set 5 #Ewmh


$set 6 #FbTkError


$set 7 #Fluxbox


$set 8 #Gnome


$set 9 #Keys


$set 10 #Menu

3 Изход
4 Икони
7 Разположение
9 Рестартиране

$set 11 #Remember


$set 12 #Screen

2 W: %4d x H: %4d
4 W: %04d x H: %04d

$set 13 #Slit

4 Posoka na Slit -a
7 Разположение на Slit -а
8 Slit

$set 14 #Toolbar

1 Редактирай името на текущото Работно Място
10 Разположение на Toolbar -а
11 Toolbar

$set 15 #Window

1 Безиме

$set 16 #Windowmenu

1 Затвори
2 Иконизирай
4 Сниши
5 Увеличи
6 Повдигни
7 Прати на ...
8 Засенчи
9 Забоди

$set 17 #Workspace

1 Работно Място %d
2 Работни Места
3 Ново Работно Място
4 Махни Последното

$set 18 #fbsetroot

1 error: must specify one of: -solid, -mod, -gradient\n
3 -display <string>        display connection\n\
-mod <x> <y>             modula pattern\n\
-foreground, -fg <color> modula foreground color\n\
-background, -bg <color> modula background color\n\n\
-gradient <texture>      gradient texture\n\
-from <color>            gradient start color\n\
-to <color>              gradient end color\n\n\
-solid <color>           solid color\n\n\
-help                    print this help text and exit\n

$set 19 #main

1 грешка: '-display' изисква аргумент
11 грешка: '-rc' изисква аргумент
13 Fluxkbox %s: (c) %s Fluxbox Team\n\n\
-display <string>\t\tuse display connection.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit\n\
-no-toolbar\t\tdo not provide a toolbar\n\
-rc <string>\t\t\tползвай заместващ ресурсен файл.\n\
-version\t\t\tпокажи версията и излез.\n\
-info\t\t\t\tdisplay some useful information.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tlog output to file.\n\
-help\t\t\t\tпокажи този помощен текст и излез.\n\n


