$ codeset=KOI8-U

$set 1 #Align

1 Знизу в центрі
2 Знизу ліворуч
3 Знизу праворуч
4 Горизонтально
5 Ліворуч
6 Ліворуч знизу
7 Ліворуч у центрі
8 Ліворуч згори
9 На всю ширину
10 Праворуч
11 Праворуч знизу
12 Праворуч у центрі
13 Праворуч угорі
14 Угорі в центрі
15 Угорі ліворуч
16 Угорі праворуч
17 Вертикально

$set 2 #BaseDisplay


$set 3 #Common
1 Прозорість
2 Автоприховання
3 Скомпільований
4 Скомпільований з опціями
5 Компілятор
6 Версія компілятора
7 Файл ресурсів
8 Файл комбінацій клавіш
10 Файл меню
11 Файл зі стилем
12 Налаштування по замовчуванню
13 вимкнена
15 Версія Fluxbox
16 Під вікнами
18 Видима

$set 4 #Configmenu
1 Згладжування шрифтів
2 Автоприховання
3 На передній план по кліку
4 Фокус по кліку
5 Прикрашати нерезидентні вікна
6 Перемикання стільниць коліщатком миші
7 Фокус на останнє вікно стільниці
8 Модель фокусування
9 Переміщувати фокус на нові вікна
10 Повна максимізація
11 Згладжувати зображення
12 Переміщення заповнених вікон
15 Переміщення вікон між стільницями
17 Меню
18 Прозорість
19 Активне вікно
20 Неактивне вікно

$set 5 #Ewmh


$set 6 #FbTkError


$set 7 #Fluxbox


$set 8 #Gnome


$set 9 #Keys


$set 10 #Menu

1 Налаштування
3 Вихід
4 Згорнуті вікна
5 Шар...
7 Розміщення
9 Перезапуск

$set 11 #Remember
1 Декорації
2 Розміри
3 Переміститися на стільницю
4 Шар
5 Запам'ятати...
6 Розташування
7 Зберігати при закритті
8 Згорнуто в заголовок
9 Приклеєний
11 Стільниця

$set 12 #Screen

2 W: %4d x H: %4d
4 W: %04d x H: %04d

$set 13 #Slit
1 Клієнти
2 По колу вниз
3 По колу вгору
4 Орієнтація докера
7 Розташування докера
8 Докер
9 Зберегти список клієнтів

$set 14 #Toolbar

1 Редагувати ім'я поточної стільниці
2 Режим відображення вікон
3 Всі вікна зі всіх стільниць
4 Мінімізовані на всіх стільницях
5 Не відображати
6 Всі вікна
7 Мінімізовані
10 Розташування панелі інструментів
11 Панель інструментів
12 Ширина у відсотках
16 Відображати іконки

$set 15 #Window

1 Безіменне

$set 16 #Windowmenu

1 Закрити
2 Згорнути на панель інструментів
3 Шар...
4 На задній план
5 Максимізувати
6 На передній план
7 Відправити вікно на ...
8 Згорнути у заголовок
9 Приклеїти

$set 17 #Workspace

1 Стільниця %d
2 Стільниці
3 Нова стільниця
4 Видалити останню

$set 18 #fbsetroot

1 помилка: необхідно задати один із таких ключів: -solid, -mod, -gradient\n
3 -display <string>        з'єднання з дисплеєм\n\
-mod <x> <y>             макет клітинки\n\
-foreground, -fg <color> колір переднього плану клітинки\n\
-background, -bg <color> колір фона клітинки\n\n\
-gradient <texture>      градієнт\n\
-from <color>            початковий колір градієнту\n\
-to <color>              кінцевий колір градієнту\n\n\
-solid <color>           суцільний колір\n\n\
-help                    вивести цю підказку і вийти\n

$set 19 #main

1 помилка: '-display' потребує аргумент
11 помилка: '-rc' потребує аргумент
13 Fluxbox %s: (c) %s Fluxbox Team\n
Website: http://www.fluxbox.org/\n\n\
-display <string>\t\tвикористовувати заданий дисплей.\n\
-screen <all|int,int,int>\trun on specified screens only.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <string>\t\t\tвикористовувати альтернативний файл ресурсів.\n\
-version\t\t\tвивести номер версії і вийти.\n\
-info\t\t\t\tвивести деяку корисну інформацію і вийти.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filename>\t\t\tжурналювати вивід у вказаний файл.\n\
-help\t\t\t\tвивести цю підказку і вийти.\n\n

$set 20 #Layer
1 Над доком
2 Знизу
3 Стільниця
4 Док
5 Нормальний
6 Згори

