$ codeset=UTF-8

$set 1 #Align

1 Nederst på midten
2 Nede til venstre
3 Nede til høyre
4 Vannrett
5 Venstre
6 Venstre nede
7 Venstre midten
8 Venstre topp
9 Relativ
10 Høyre
11 Høyre nede
12 Høyre midten
13 Høyre oppe
14 Øverst på midten
15 Øverst til venstre
16 Øverst til høyre
17 Loddrett

$set 2 #BaseDisplay

1 Avbryter . . . dumper kjerne\n
2 Slår av\n
3 %s: fikk signal %d\n

$set 3 #Common

1 Gjennomsiktighet
2 Skjul automatisk
3 Bygget
4 Byggete alternativer
5 Bygger
6 Byggerversjon
7 init
8 taster
9 oversetting
10 meny
11 stil
12 Standarder
13 slått av
14 Feil
15 FluxBox-versjon
16 Maksimer over
17 GIT-revisjon
18 Synlig

$set 4 #Configmenu

1 Kantutjevning
2 Hev automatisk
3 Klikk hever
4 Klikk for fokus
5 Dekorer kortvarige vinduer
6 Musehjul bytter skrivebord
7 Fokus på vindu ved endring av arbeidsområde
8 Fokus-modell
9 Fokus på nye vinduer
10 Full maksimering
11 Bildeutjevning
12 Flytt vinduer ugjennomsiktig
14 Musefokus
15 Bøying av arbeidsområde
16 Tving halvgjennomsiktighet
17 Meny-alfa
18 Gjennomsiktighet
19 Gjennomsiktighet for vindu i fokus
20 Gjennomsiktighet for vindu ute av fokus

$set 5 #Ewmh

1 Fatalt: Ikke mer minne; klarte ikke finne nok for EWMH-klientliste

$set 6 #FbTkError

1 Advarsel! Høyde > 3200, setter høyde til 3200
2 Advarsel! Bredde > 3200, setter bredde til 3200
3 Advarsel: klarte ikke laste tilbakefallskriften
4 Tildelingsfeil.
5 Klarte ikke lage pikselkart
6 Klarte ikke lage «XImage»
7 Klarte ikke lese
8 Advarsel: Klarte ikke finne gyldig format for alfa.
9 Advarsel: klarte ikke lage alfabilde.
10 Advarsel: klarte ikke lage alfa-pikselkart.
11 Klarte ikke finne format for skjerm (%d)
12 Ikke mer minne
13 Tom for minne ved tildeling av hurtigminen for blå.
14 Tom for minne ved tildeling av hurtigminen for grønn.
15 Tom for minne ved tildeling av hurtigminen for rød.
16 Klarte ikke lese temaelement
17 visuell støttes ikke
18 Angir standardverdi

$set 7 #Fluxbox

1 Ressursfilnavnet er ugyldig.
2 Klarte ikke laste gruppefil
3 Klarte ikke laste database
4 Prøver på nytt med
5 Advarsel! Klarte ikke finne skjerm å avbilde vindu på.
6 Klarte ikke lage katalogen «%s»
7 Klarte ikke finne skjermer å kontrollere.\nSørg for at du ikke har en annen vindusbehandler kjørende.
8 Klarte ikke spalte regulært uttrykk
9 Fatalt! Det kan kun være én instans av FluxBox-klassen.
10 Klarte ikke koble til X-tjener.\nForsikre deg om at du startet X før du startet FluxBox.
11 Advarsel: X-tjeneren støtter ikke lokaliteten
12 Advarsel: klarte ikke velge lokalitetmodifikatorer

$set 8 #Gnome

1 Fatalt: ikke mer minne; kan ikke tildele for GNOME-klientliste

$set 9 #Keys

1 Taster: feil på linje
2 Taster: klarte ikke slå sammen tastetre.
3 Taster: ugyldig tast/modifikator på linje

$set 10 #Menu

1 Oppsett
2 FluxBox standardmeny
3 Avslutt
4 Ikoner
5 Lag . . .
6 På Hode . . .
7 Plassering
8 Last oppsett på nytt
9 Start på nytt

$set 11 #Remember

1 Dekorasjoner
2 Dimensjoner
3 Hopp til arbeidsområde
4 Lag
5 Husk . . .
6 Posisjon
7 Lagre ved avslutting
8 Skyggelagt
9 Klebrig
10 Ukjent programtast
11 Arbeidsområde
12 Hode

$set 12 #Screen

1 BScreen::BScreen: feil ved spørring av X-tjeneren;\n	en annen vindusbehandler kjører allerede på visningsenheten
2 B: %4d x H: %4d
3 BScreen::BScreen: kontrollerer skjerm %d ved hjelp av visuell 0x%lx, dybde %d\n

$set 13 #Slit

1 Klienter
2 Syklus ned
3 Syklus opp
4 Spalteretning
5 Spaltelag
6 Spalt på Hode
7 Spalteplassering
8 Spalt
9 Lagre spalteliste

$set 14 #Toolbar

1 Rediger navnet på gjeldende arbeidsområde
2 Ikonlinje-modus
3 Alle vinduer
4 Ikoner
5 Ingen
6 Arbeidsområde
7 Arbeidsområde: ikoner
8 Verktøylinjelag
9 Vertkøylinje på Hode
10 Plassering av verktøylinje
11 Verktøylinje
12 Bredde på verktøylinje i prosent
13 Klokke: 24t
14 Klokke: 12t
15 Rediger klokkeformat
16 Vis bilder
17 Ingen ikoner
18 Arbeidsområde: ingen ikoner

$set 15 #Window

1 Uten navn

$set 16 #Windowmenu

1 Lukk
2 Ikonifiser
3 Lag
4 Senk
5 Maksimer
6 Hev
7 Send til . . .
8 Skyggelegg
9 Fest
10 Drep

$set 17 #Workspace

1 Arbeidsområde %d
2 Arbeidsområder
3 Nytt arbeidsområde
4 Fjern siste

$set 18 #fbsetroot

1 feil: må oppgi en av: -solid, -mod, -gradient\n
2 Klarte ikke opprette bildekartatomer, gir opp.
3 -display <streng>        tilkobling til visningsenhet\n\
-mod <x> <y>             «modula»-mønster\n\
-foreground, -fg <farge> «modula»-forgrunnsfarge\n\
-background, -bg <farge> «modula»-bakgrunnsfarge\n\n\
-gradient <tekstur>      overgangtekstur\n\
-from <farge>            startfarge for overgang\n\
-to <farge>              sluttfarge for overgang\n\n\
-solid <farge>           ensfarget\n\n\
-help                    hvis denne hjelpeteksten og avslutt\n

$set 19 #main

1 feil: «-display» krever et argument
2 Feil tildeling
3 Feil typeomforming
4 Utenfor rekkevidde
5 Kjøretidsfeil
6 Standard Exception
7 Ukjent feil
8 feil: «-log» trenger et argument
9 Loggfil
10 Logger til
11 feil: «-rc» krever et argument
12 feil: «-rc» krever et argument
13 FluxBox %s: kopirett %s Fluxbox Team\n\
Nettside: http://www.fluxbox.org/\n\n\
-display <streng>\t\tbruk visningsenhetforbindelse.\n\
-screen <alle|int,int,int>\tkjør kun på valgte skjermer.\n\
-no-slit\t\t\tdo not provide a slit.\n\
-no-toolbar\t\t\tdo not provide a toolbar.\n\
-rc <streng>\t\t\tbruk alternativ ressursfil.\n\
-version\t\t\tvis versjonnummer og avslutt.\n\
-info\t\t\t\tvis litt nyttig informasjon.\n\
-list-commands\t\t\tlist all valid key commands.\n\
-sync\t\t\t\tsynchronize with X server for debugging.\n\
-log <filnavn>\t\t\tlogg utdata til fil.\n\
-help\t\t\t\tvis denne hjelpeteksten og avslutt.\n\n
14 advarsel: klarte ikke angi miljøvariabelen «DISPLAY»

$set 20

1 Over feste
2 Bunn
3 Skrivebord
4 Feste
5 Normal
6 Topp



 	  	 
