$ codeset=UTF-8

$set 1 #Align

1 하단 중앙
2 하단 왼쪽
3 하단 오른쪽
4 [수평]
5 왼쪽 정렬
6 좌측 아래
7 좌측 중간
8 좌측 위
9 양쪽 정렬
10 오른쪽 정렬
11 우측 아래
12 우측 중간
13 우측 위
14 상단 중앙
15 상단 왼쪽
16 상단 오른쪽
17 [수직]

$set 2 #BaseDisplay


$set 3 #Common

1 투명도
2 자동 숨김
16 창 최대화 시 덮임
18 툴바 표시
19 이 스타일에는 바탕화면 항목이 명시되어 있지 않다.\n메뉴얼을 참고하거나 FAQ를 읽으시오.

$set 4 #Configmenu

1 안티 앨리어스
2 초점 가진 창 자동 올림
3 클릭 시 창 올림
4 클릭 시 초점 가짐
5 일시적인 창도 테두리 꾸밈
6 마우스 휠로 작업공간 이동
7 작업공간 전환 시 초점 가진 창 기억
8 활성화 방법
9 새로 여는 창 초점 가짐
10 화면 전체를 덮는 창 최대화
11 이미지 디더링
12 창 이동 시 내용 표시
14 커서 아래 창 초점 가짐
15 작업공간간 창 드래그 이동 
16 가짜 투명 강제 사용
17 메뉴 투명도
18 투명도 설정
19 초점 가진 창 투명도
20 초점 없는 창 투명도
21 탭 옵션
22 타이틀바에 탭 내장
23 외부탭 너비
24 커서 아래 탭 초점 가짐
25 클릭 시 탭 초점 가짐

$set 5 #Ewmh


$set 6 #FbTkError


$set 7 #Fluxbox


$set 8 #Gnome


$set 9 #Keys


$set 10 #Menu

1 설정
2 플럭스박스 default 메뉴
3 종료
4 숨긴 창
5 레이어 ...
6 위치 모니터 ...
7 위치
8 설정을 다시 읽어들임
9 재시작

$set 11 #Remember

1 테두리 상태
2 크기
3 기억한 작업공간으로 전환
4 레이어
5 기억 ...
6 위치
7 종료 시 값들 기억
8 셰이드 여부
9 고정 여부
11 작업공간
12 위치 모니터
13 투명도

$set 12 #Screen


$set 13 #Slit

1 클라이언트
2 한 칸씩 내림
3 한 칸씩 올림
4 Slit 방향
5 Slit 레이어
6 Slit 위치 모니터
7 Slit 위치
9 SlitList 저장

$set 14 #Toolbar

1 작업공간 이름 변경
2 아이콘바 모드
3 모든 작업공간 창 표시
4 숨긴 창 표시
5 아무 것도 표시 안 함
6 현재 작업공간 창 표시
7 현재 작업공간 숨긴 창 표시
8 툴바 레이어
9 툴바 위치 모니터
10 툴바 위치
11 툴바
12 툴바 너비(%)
13 시계: 24 시간제
14 시계: 12 시간제
15 시간 표시 형식 수정
16 아이콘 이미지 표시
17 숨긴 창 빼고 표시
18 현재 작업공간 숨긴 창 빼고 표시

$set 15 #Window


$set 16 #Windowmenu

1 닫음
2 숨김
3 레이어
4 낮춤
5 최대화
6 올림
7 보냄 ...
8 셰이드
9 고정
10 강제 종료
11 기본 투명도 

$set 17 #Workspace

1 작업공간 %d
2 작업공간
3 새 작업공간 추가
4 끝에것 제거

$set 18 #bsetroot


$set 19 #main


$set 20

1 Dock 위
2 하부
3 데스크톱
5 기본
6 상위


